@@ -23,9 +23,9 @@
 #ifndef __THUNAR_APPLICATION_H__
 #define __THUNAR_APPLICATION_H__
 
-#include <thunar/thunar-job-operation.h>
-#include <thunar/thunar-window.h>
-#include <thunar/thunar-thumbnail-cache.h>
+#include "thunar/thunar-job-operation.h"
+#include "thunar/thunar-window.h"
+#include "thunar/thunar-thumbnail-cache.h"
 
 G_BEGIN_DECLS;
 
@@ -139,12 +139,12 @@ void                  thunar_application_move_files                (ThunarApplic
                                                                     ThunarOperationLogMode   log_mode,
                                                                     GClosure                *new_files_closure);
 
-gboolean              thunar_application_unlink_files              (ThunarApplication       *application,
-                                                                    gpointer                 parent,
-                                                                    GList                   *file_list,
-                                                                    gboolean                 permanently,
-                                                                    gboolean                 warn);
-
+gboolean              thunar_application_unlink_files              (ThunarApplication           *application,
+                                                                    gpointer                     parent,
+                                                                    GList                       *file_list,
+                                                                    gboolean                     permanently,
+                                                                    gboolean                     warn,
+                                                                    const ThunarOperationLogMode log_mode);
 
 void                  thunar_application_trash                     (ThunarApplication       *application,
                                                                     gpointer                 parent,
