@@ -19,16 +19,18 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <glib-object.h>
 
-#include <thunar/thunar-browser.h>
-#include <thunar/thunar-file.h>
-#include <thunar/thunar-private.h>
-#include <thunar/thunar-util.h>
-#include <thunar/thunar-gtk-extensions.h>
+#include <libxfce4util/libxfce4util.h>
+
+#include "thunar/thunar-browser.h"
+#include "thunar/thunar-file.h"
+#include "thunar/thunar-private.h"
+#include "thunar/thunar-util.h"
+#include "thunar/thunar-gtk-extensions.h"
 
 
 
@@ -518,7 +520,7 @@ thunar_browser_poke_file_internal (ThunarBrowser                *browser,
  * In the other cases, if the enclosing volume of the @file is not mounted
  * yet, it tries to mount it.
  *
- * When finished or if an error occured, it calls @func with the provided
+ * When finished or if an error occurred, it calls @func with the provided
  * @user data. The #GError parameter to @func is set if, and only if there
  * was an error.
  **/
