@@ -21,17 +21,17 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
-#include <thunar/thunar-chooser-button.h>
-#include <thunar/thunar-chooser-dialog.h>
-#include <thunar/thunar-dialogs.h>
-#include <thunar/thunar-gobject-extensions.h>
-#include <thunar/thunar-gtk-extensions.h>
-#include <thunar/thunar-icon-factory.h>
-#include <thunar/thunar-pango-extensions.h>
-#include <thunar/thunar-private.h>
+#include "thunar/thunar-chooser-button.h"
+#include "thunar/thunar-chooser-dialog.h"
+#include "thunar/thunar-dialogs.h"
+#include "thunar/thunar-gobject-extensions.h"
+#include "thunar/thunar-gtk-extensions.h"
+#include "thunar/thunar-icon-factory.h"
+#include "thunar/thunar-pango-extensions.h"
+#include "thunar/thunar-private.h"
 
 
 
@@ -305,9 +305,15 @@ thunar_chooser_button_changed (GtkComboBox *combo_box)
       if (!g_app_info_set_as_default_for_type (app_info, content_type, &error))
         {
           /* tell the user that it didn't work */
-          thunar_dialogs_show_error (GTK_WIDGET (chooser_button), error,
-                                     _("Failed to set default application for \"%s\""),
-                                     thunar_file_get_display_name (chooser_button->file));
+          if (g_strcmp0 (thunar_file_get_display_name (chooser_button->file), thunar_file_get_basename (chooser_button->file)) != 0)
+            thunar_dialogs_show_error (GTK_WIDGET (chooser_button), error,
+                                       _("Failed to set default application for \"%s\" (%s)"),
+                                       thunar_file_get_display_name (chooser_button->file),
+                                       thunar_file_get_basename (chooser_button->file));
+          else
+            thunar_dialogs_show_error (GTK_WIDGET (chooser_button), error,
+                                       _("Failed to set default application for \"%s\""),
+                                       thunar_file_get_display_name (chooser_button->file));
           g_error_free (error);
         }
       else
