@@ -20,7 +20,7 @@
 #ifndef __THUNAR_DETAILS_VIEW_H__
 #define __THUNAR_DETAILS_VIEW_H__
 
-#include <thunar/thunar-standard-view.h>
+#include "thunar/thunar-standard-view.h"
 
 G_BEGIN_DECLS;
 
@@ -38,6 +38,7 @@ typedef struct _ThunarDetailsView      ThunarDetailsView;
 typedef enum
 {
   THUNAR_DETAILS_VIEW_ACTION_CONFIGURE_COLUMNS,
+  THUNAR_DETAILS_VIEW_ACTION_TOGGLE_EXPANDABLE_FOLDERS,
 } ThunarDetailsViewAction;
 
 GType      thunar_details_view_get_type                         (void) G_GNUC_CONST;
