@@ -376,14 +376,25 @@ thunar_device_get_name (const ThunarDevice *device)
 
 
 GIcon *
-thunar_device_get_icon (const ThunarDevice *device)
+thunar_device_get_icon (const ThunarDevice *device,
+                        gboolean            symbolic)
 {
   _thunar_return_val_if_fail (THUNAR_IS_DEVICE (device), NULL);
 
   if (G_IS_VOLUME (device->device))
-    return g_volume_get_icon (device->device);
+    {
+      if (symbolic)
+        return g_volume_get_symbolic_icon (device->device);
+      else
+        return g_volume_get_icon (device->device);
+    }
   else if (G_IS_MOUNT (device->device))
-   return g_mount_get_icon (device->device);
+    {
+      if (symbolic)
+        return g_mount_get_symbolic_icon (device->device);
+      else
+        return g_mount_get_icon (device->device);
+    }
   else
     _thunar_assert_not_reached ();
 
@@ -526,11 +537,11 @@ thunar_device_get_eject_label (const ThunarDevice *device)
     switch (start_stop_type)
       {
       case G_DRIVE_START_STOP_TYPE_SHUTDOWN:
-        return _("_Safely Remove");
+        return _("Safely _Remove");
       case G_DRIVE_START_STOP_TYPE_NETWORK:
         return _("_Disconnect");
       case G_DRIVE_START_STOP_TYPE_MULTIDISK:
-        return _("_Stop the Multi-Disk Drive");
+        return _("Stop the _Multi-Disk Drive");
       case G_DRIVE_START_STOP_TYPE_PASSWORD:
         return _("_Lock");
       case G_DRIVE_START_STOP_TYPE_UNKNOWN:
