@@ -293,6 +293,7 @@ emblem_change_job_finished (gpointer data)
     thunar_window_queue_redraw (lp->data);
   g_list_free (windows);
 
+  g_object_unref (chooser->emblem_change_job);
   chooser->emblem_change_job = NULL;
   chooser->emblem_change_job_finish_signal = 0;
 }
@@ -335,6 +336,28 @@ thunar_emblem_chooser_button_toggled (GtkToggleButton     *button,
     }
   g_list_free (children);
 
+  /* limit the number of selectable emblems */
+  if (g_list_length (emblem_names) > MAX_EMBLEMS_PER_FILE)
+    {
+      GtkWidget *toplevel;
+      GtkWindow *window = NULL;
+
+      /* determine the toplevel window for the chooser */
+      toplevel = gtk_widget_get_toplevel (GTK_WIDGET (chooser));
+      if (toplevel != NULL)
+        window = GTK_WINDOW (toplevel);
+
+      g_signal_handlers_block_by_func (G_OBJECT (button), thunar_emblem_chooser_button_toggled, chooser);
+      gtk_toggle_button_set_active (button, FALSE);
+      g_signal_handlers_unblock_by_func (G_OBJECT (button), thunar_emblem_chooser_button_toggled, chooser);
+
+      gchar* message = g_strdup_printf (_("A maximum of %u emblems is supported per file."), MAX_EMBLEMS_PER_FILE);
+      xfce_dialog_show_warning (window, message, _("Too many emblems selected"));
+      g_free (message);
+      g_list_free (emblem_names);
+      return;
+    }
+
   /* Disable listening to the "changed" signal of the files to prevent lag */
   for (lp = chooser->files; lp != NULL; lp = lp->next)
     g_signal_handlers_block_by_func (lp->data, thunar_emblem_chooser_file_changed, chooser);
@@ -383,6 +406,7 @@ thunar_emblem_chooser_file_changed (ThunarEmblemChooser *chooser)
   GObject     *child;
   guint       *count;
   guint        n_files = 0;
+  guint        n_emblems = 0;
 
   _thunar_return_if_fail (THUNAR_IS_EMBLEM_CHOOSER (chooser));
 
@@ -392,8 +416,12 @@ thunar_emblem_chooser_file_changed (ThunarEmblemChooser *chooser)
   for (lp = chooser->files; lp != NULL; lp = lp->next)
     {
       file_emblems = thunar_file_get_emblem_names (THUNAR_FILE (lp->data));
-      for (li = file_emblems; li != NULL; li = li->next)
+      for (n_emblems = 0,li = file_emblems; li != NULL; li = li->next, n_emblems++)
         {
+          /* dont load more emblems than we can display */
+          if (n_emblems >= MAX_EMBLEMS_PER_FILE)
+            break;
+
           count = g_hash_table_lookup (emblem_names, li->data);
           if (count == NULL)
             {
