@@ -220,6 +220,7 @@ ThunarFileMode    thunar_file_get_mode                   (const ThunarFile
 gboolean          thunar_file_is_mounted                 (const ThunarFile       *file);
 gboolean          thunar_file_exists                     (const ThunarFile       *file);
 gboolean          thunar_file_is_directory               (const ThunarFile       *file) G_GNUC_PURE;
+gboolean          thunar_file_is_empty_directory         (const ThunarFile       *file) G_GNUC_PURE;
 gboolean          thunar_file_is_shortcut                (const ThunarFile       *file) G_GNUC_PURE;
 gboolean          thunar_file_is_mountable               (const ThunarFile       *file) G_GNUC_PURE;
 gboolean          thunar_file_is_local                   (const ThunarFile       *file);
@@ -239,6 +240,7 @@ gboolean          thunar_file_is_trash                   (const ThunarFile
 gboolean          thunar_file_is_trashed                 (const ThunarFile       *file);
 gboolean          thunar_file_is_recent                  (const ThunarFile       *file);
 gboolean          thunar_file_is_in_recent               (const ThunarFile       *file);
+void              thunar_file_add_to_recent              (const ThunarFile       *file);
 gboolean          thunar_file_is_desktop_file            (const ThunarFile       *file);
 const gchar      *thunar_file_get_display_name           (const ThunarFile       *file) G_GNUC_CONST;
 
@@ -256,7 +258,7 @@ gboolean          thunar_file_is_renameable              (const ThunarFile
 gboolean          thunar_file_can_be_trashed             (const ThunarFile       *file);
 
 
-guint             thunar_file_get_file_count             (ThunarFile             *file,
+gint             thunar_file_get_file_count              (ThunarFile             *file,
                                                           GCallback               callback,
                                                           gpointer                data);
 void              thunar_file_set_file_count             (ThunarFile             *file,
@@ -269,6 +271,8 @@ gboolean          thunar_file_set_custom_icon            (ThunarFile
                                                           const gchar             *custom_icon,
                                                           GError                 **error);
 
+void              thunar_file_set_is_thumbnail           (ThunarFile              *file,
+                                                          gboolean                 is_thumbnail);
 const gchar         *thunar_file_get_thumbnail_path      (ThunarFile              *file,
                                                           ThunarThumbnailSize      thumbnail_size);
 ThunarFileThumbState thunar_file_get_thumb_state         (const ThunarFile        *file,
@@ -370,6 +374,7 @@ void              thunar_file_request_thumbnail                 (ThunarFile
 void              thunar_file_update_thumbnail                  (ThunarFile          *file,
                                                                  ThunarFileThumbState state,
                                                                  ThunarThumbnailSize  size);
+void              thunar_file_changed                           (ThunarFile          *file);  
 
 /**
  * thunar_file_is_root:
@@ -414,19 +419,6 @@ void              thunar_file_update_thumbnail                  (ThunarFile
  **/
 #define thunar_file_has_uri_scheme(file, uri_scheme) (g_file_has_uri_scheme (thunar_file_get_file (file), (uri_scheme)))
 
-/**
- * thunar_file_changed:
- * @file : a #ThunarFile instance.
- *
- * Emits the ::changed signal on @file. This function is meant to be called
- * by derived classes whenever they notice changes to the @file.
- **/
-#define thunar_file_changed(file)                         \
-G_STMT_START{                                             \
-  thunarx_file_info_changed (THUNARX_FILE_INFO ((file))); \
-}G_STMT_END
-
-
 G_END_DECLS;
 
 #endif /* !__THUNAR_FILE_H__ */
