@@ -39,7 +39,7 @@ GType         thunar_folder_get_type               (void) G_GNUC_CONST;
 ThunarFolder *thunar_folder_get_for_file           (ThunarFile         *file);
 
 ThunarFile   *thunar_folder_get_corresponding_file (const ThunarFolder *folder);
-GList        *thunar_folder_get_files              (const ThunarFolder *folder);
+GHashTable   *thunar_folder_get_files              (const ThunarFolder *folder);
 gboolean      thunar_folder_get_loading            (const ThunarFolder *folder);
 gboolean      thunar_folder_has_folder_monitor     (const ThunarFolder *folder);
 
