@@ -18,7 +18,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #ifdef HAVE_ERRNO_H
@@ -34,8 +34,8 @@
 #include <string.h>
 #endif
 
-#include <thunar/thunar-gdk-extensions.h>
-#include <thunar/thunar-private.h>
+#include "thunar/thunar-gdk-extensions.h"
+#include "thunar/thunar-private.h"
 
 
 
