@@ -687,6 +687,38 @@ thunar_g_file_list_get_type (void)
 
 
 
+static GHashTable*
+thunar_g_hastable_copy_deep (GHashTable *table)
+{
+  GHashTable    *copy = g_hash_table_new_full (g_direct_hash, NULL, g_object_unref, NULL);
+  GHashTableIter iter;
+  gpointer       key;
+
+  g_hash_table_iter_init (&iter, table);
+  while (g_hash_table_iter_next (&iter, &key, NULL))
+    g_hash_table_add (copy, g_object_ref (key));
+  return copy;
+}
+
+
+
+GType
+thunar_g_file_hastable_get_type (void)
+{
+  static GType type = G_TYPE_INVALID;
+
+  if (G_UNLIKELY (type == G_TYPE_INVALID))
+    {
+      type = g_boxed_type_register_static (I_("ThunarGHashTableList"),
+                                           (GBoxedCopyFunc) thunar_g_hastable_copy_deep,
+                                           (GBoxedFreeFunc) g_hash_table_destroy);
+    }
+
+  return type;
+}
+
+
+
 /**
  * thunar_g_file_copy:
  * @source                 : input #GFile
@@ -1409,36 +1441,6 @@ thunar_g_file_get_link_path_for_symlink (GFile *file_to_link,
 
 
 
-gboolean
-thunar_g_file_is_empty (GFile *file)
-{
-  GFileEnumerator *enumerator;
-  GError          *err = NULL;
-  gboolean         is_empty = TRUE;
-  GFileInfo       *info;
-
-  _thunar_return_val_if_fail (G_IS_FILE (file), TRUE);
-
-  enumerator = g_file_enumerate_children (file, NULL,
-                                          G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
-                                          NULL, &err);
-
-  if (err != NULL)
-    {
-      g_warning ("Failed to check if the file has any children!");
-      return TRUE;
-    }
-
-  if (g_file_enumerator_iterate (enumerator, &info, NULL, NULL, &err) && info != NULL)
-    is_empty = FALSE;
-
-  g_object_unref (enumerator);
-
-  return is_empty;
-}
-
-
-
 static GFileInfo*
 thunar_g_file_get_content_type_querry_info (GFile *gfile,
                                             GError *err)
@@ -1578,11 +1580,13 @@ thunar_g_file_info_set_attribute (GFileInfo   *info,
         break;
 
       case THUNAR_GTYPE_STRINGV:
+      {
         gchar **setting_values;
         setting_values = g_strsplit (setting_value, THUNAR_METADATA_STRING_DELIMETER, 100);
         g_file_info_set_attribute_stringv (info, setting_name, setting_values);
         g_strfreev (setting_values);
         break;
+      }
 
       default:
         g_warning ("ThunarGType not supported, skipping");
@@ -1603,6 +1607,7 @@ thunar_g_file_info_get_attribute (GFileInfo   *info,
         return g_strdup (g_file_info_get_attribute_string (info, setting_name));
 
       case THUNAR_GTYPE_STRINGV:
+      {
         gchar **stringv = g_file_info_get_attribute_stringv (info, setting_name);
         GList  *string_list = NULL;
         gchar  *joined_string = NULL;
@@ -1616,6 +1621,7 @@ thunar_g_file_info_get_attribute (GFileInfo   *info,
         joined_string = thunar_util_strjoin_list (string_list, THUNAR_METADATA_STRING_DELIMETER);
         g_list_free (string_list);
         return joined_string;
+      }
 
       default:
         g_warning ("ThunarGType not supported, skipping");
@@ -1760,4 +1766,4 @@ thunar_g_file_get_metadata_setting (GFile       *file,
   g_free (attr_name);
 
   return attr_value;
-}
\ No newline at end of file
+}
