@@ -20,7 +20,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #ifdef HAVE_MEMORY_H
@@ -33,7 +33,7 @@
 #include <string.h>
 #endif
 
-#include <thunar/thunar-gobject-extensions.h>
+#include "thunar/thunar-gobject-extensions.h"
 
 #include <gio/gio.h>
 
