@@ -110,6 +110,8 @@ thunar_gtk_menu_thunarx_menu_item_new (GObject      *thunarx_menu_item,
                                                 G_CALLBACK (thunarx_menu_item_activate),
                                                 G_OBJECT (thunarx_menu_item), image, menu_to_append_item);
 
+  gtk_widget_set_sensitive (GTK_WIDGET (gtk_menu_item), sensitive);
+
   /* recursively add submenu items if any */
   if (gtk_menu_item != NULL && thunarx_menu != NULL)
   {
@@ -372,3 +374,22 @@ thunar_gtk_editable_can_paste (GtkEditable *editable)
 {
   return gtk_editable_get_editable (editable);
 }
+
+
+
+/**
+ * thunar_gtk_orientable_get_center_pos:
+ *
+ * Return value: The pixel corrsponding to the center of the orientable widget (either in height or in width) 
+ **/
+gint
+thunar_gtk_orientable_get_center_pos (GtkOrientable *orientable)
+{
+  GtkAllocation allocation;
+
+  gtk_widget_get_allocation (GTK_WIDGET (orientable), &allocation);
+  if (gtk_orientable_get_orientation (orientable) == GTK_ORIENTATION_HORIZONTAL)
+    return (gint)(allocation.width/2);
+  else
+    return (gint)(allocation.height/2);
+}
