@@ -156,6 +156,9 @@ thunar_history_finalize (GObject *object)
   g_slist_free_full (history->forward_list, g_object_unref);
   g_slist_free_full (history->back_list, g_object_unref);
 
+  if (history->current_directory != NULL)
+    g_object_unref (history->current_directory);
+
   (*G_OBJECT_CLASS (thunar_history_parent_class)->finalize) (object);
 }
 
@@ -569,7 +572,8 @@ thunar_history_show_menu (ThunarHistory         *history,
       if (file != NULL)
         {
           /* load the icon for the file */
-          icon = thunar_icon_factory_load_file_icon (icon_factory, file, THUNAR_FILE_ICON_STATE_DEFAULT, 16, scale_factor);
+          icon = thunar_icon_factory_load_file_icon (icon_factory, file, THUNAR_FILE_ICON_STATE_DEFAULT, 16,
+                                                     scale_factor, FALSE, NULL);
           if (icon != NULL)
             {
               /* setup the image for the file */
