@@ -43,6 +43,7 @@ enum
   PROP_ROUNDED_CORNERS,
   PROP_HIGHLIGHTING_ENABLED,
   PROP_IMAGE_PREVIEW_ENABLED,
+  PROP_USE_SYMBOLIC_ICONS,
 };
 
 
@@ -210,6 +211,19 @@ thunar_icon_renderer_class_init (ThunarIconRendererClass *klass)
                                    g_param_spec_boolean ("image-preview-enabled", "image-preview-enabled", "image-preview-enabled",
                                                          FALSE,
                                                          EXO_PARAM_READWRITE));
+
+  /**
+   * ThunarIconRenderer:use_symbolic_icons:
+   *
+   * Whether to use symbolic icons.
+   **/
+  g_object_class_install_property (gobject_class,
+                                   PROP_USE_SYMBOLIC_ICONS,
+                                   g_param_spec_boolean ("use-symbolic-icons",
+                                                         "use-symbolic-icons",
+                                                         "use-symbolic-icons",
+                                                         FALSE,
+                                                         EXO_PARAM_READWRITE));
 }
 
 
@@ -285,6 +299,10 @@ thunar_icon_renderer_get_property (GObject    *object,
     case PROP_IMAGE_PREVIEW_ENABLED:
       g_value_set_boolean (value, icon_renderer->highlighting_enabled);
       break;
+ 
+    case PROP_USE_SYMBOLIC_ICONS:
+      g_value_set_boolean (value, icon_renderer->use_symbolic_icons);
+      break;
 
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -345,6 +363,10 @@ thunar_icon_renderer_set_property (GObject      *object,
       icon_renderer->image_preview_enabled = g_value_get_boolean (value);
       break;
 
+    case PROP_USE_SYMBOLIC_ICONS:
+      icon_renderer->use_symbolic_icons = g_value_get_boolean (value);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -471,6 +493,7 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
   ThunarIconRenderer     *icon_renderer = THUNAR_ICON_RENDERER (renderer);
   ThunarIconFactory      *icon_factory;
   GtkIconTheme           *icon_theme;
+  GtkStyleContext        *context = NULL;
   GdkRectangle            emblem_area;
   GdkRectangle            icon_area;
   GdkRectangle            clip_area;
@@ -480,7 +503,6 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
   GList                  *emblems;
   GList                  *lp;
   gint                    scale_factor;
-  gint                    max_emblems;
   gint                    position;
   gdouble                 alpha;
   gint                    emblem_size;
@@ -509,13 +531,24 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
   /* load the main icon */
   icon_theme = gtk_icon_theme_get_for_screen (gtk_widget_get_screen (widget));
   icon_factory = thunar_icon_factory_get_for_icon_theme (icon_theme);
-  scale_factor = gtk_widget_get_scale_factor (widget);
-  thunar_file_request_thumbnail (icon_renderer->file, thunar_icon_size_to_thumbnail_size (icon_renderer->size * scale_factor));
 
+  scale_factor = gtk_widget_get_scale_factor (widget);
+
+  /* load thumbnail for supported file type */
+  if (thunar_icon_factory_get_show_thumbnail (icon_factory, icon_renderer->file))
+    thunar_file_request_thumbnail (icon_renderer->file, thunar_icon_size_to_thumbnail_size (icon_renderer->size * scale_factor));
+
+  /* load additional thumbnail for image preview */
   if (icon_renderer->image_preview_enabled)
     thunar_file_request_thumbnail (icon_renderer->file, THUNAR_THUMBNAIL_SIZE_XX_LARGE);
 
-  icon = thunar_icon_factory_load_file_icon (icon_factory, icon_renderer->file, icon_state, icon_renderer->size, scale_factor);
+  /* get style context for symbolic icon */
+  if (icon_renderer->use_symbolic_icons)
+    context = gtk_widget_get_style_context (widget);
+
+  icon = thunar_icon_factory_load_file_icon (icon_factory, icon_renderer->file, icon_state,
+                                             icon_renderer->size, scale_factor,
+                                             icon_renderer->use_symbolic_icons, context);
   if (G_UNLIKELY (icon == NULL))
     {
       g_object_unref (G_OBJECT (icon_factory));
@@ -598,17 +631,15 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
       emblems = thunar_file_get_emblem_names (icon_renderer->file);
       if (G_UNLIKELY (emblems != NULL))
         {
-          /* render up to four emblems for sizes from 48 onwards, else up to 2 emblems */
-          max_emblems = (icon_renderer->size < 48) ? 2 : 4;
-
-          /* render the emblems */
-          for (lp = emblems, position = 0; lp != NULL && position < max_emblems; lp = lp->next)
+          /* render up to MAX_EMBLEMS_PER_FILE emblems */
+          for (lp = emblems, position = 0; lp != NULL && position < MAX_EMBLEMS_PER_FILE; lp = lp->next)
             {
               /* calculate the emblem size */
-              emblem_size = MIN ((2 * icon_renderer->size) / 3, 32);
+              emblem_size = MIN ((2 * icon_renderer->size) / 4, 32);
 
               /* check if we have the emblem in the icon theme */
-              emblem = thunar_icon_factory_load_icon (icon_factory, lp->data, emblem_size, scale_factor, FALSE);
+              emblem = thunar_icon_factory_load_icon (icon_factory, lp->data, emblem_size, scale_factor, FALSE,
+                                                      icon_renderer->use_symbolic_icons, context);
               if (G_UNLIKELY (emblem == NULL))
                 continue;
 
