@@ -1581,7 +1581,9 @@ _thunar_search_folder (ThunarStandardViewModel           *model,
       /* handle directories */
       if (type == G_FILE_TYPE_DIRECTORY && search_type == THUNAR_STANDARD_VIEW_MODEL_SEARCH_RECURSIVE)
         {
-          _thunar_search_folder (model, job, g_file_get_uri (file), search_query_c_terms, search_type, show_hidden);
+          gchar *file_uri = g_file_get_uri (file);
+          _thunar_search_folder (model, job, file_uri, search_query_c_terms, search_type, show_hidden);
+          g_free (file_uri);
         }
 
       /* prepare entry display name */
@@ -1848,21 +1850,26 @@ _thunar_job_load_content_types (ThunarJob *job,
                                 GArray    *param_values,
                                 GError   **error)
 {
-  GList *thunar_files;
+  GHashTable    *thunar_files;
+  ThunarFile    *file;
+  gpointer       key;
+  GHashTableIter iter;
 
   if (exo_job_set_error_if_cancelled (EXO_JOB (job), error))
     return FALSE;
 
   thunar_files = g_value_get_boxed (&g_array_index (param_values, GValue, 0));
 
-  for (GList *lp = thunar_files; lp != NULL; lp = lp->next)
+  g_hash_table_iter_init (&iter, thunar_files);
+  while (g_hash_table_iter_next (&iter, &key, NULL))
     {
       gchar *content_type;
       GFile       *g_file;
 
-      g_file = thunar_file_get_file (THUNAR_FILE (lp->data));
+      file = THUNAR_FILE (key);
+      g_file = thunar_file_get_file (file);
       content_type = thunar_g_file_get_content_type (g_file);
-      thunar_file_set_content_type (THUNAR_FILE (lp->data), content_type);
+      thunar_file_set_content_type (file, content_type);
       g_free (content_type);
     }
 
@@ -1882,12 +1889,147 @@ _thunar_job_load_content_types (ThunarJob *job,
  * Returns: (transfer none): the #ThunarJob which manages the separate thread
  **/
 ThunarJob *
-thunar_io_jobs_load_content_types (GList *files)
+thunar_io_jobs_load_content_types (GHashTable *files)
 {
-  GList     *copy = thunar_g_list_copy_deep (files);
   ThunarJob *job = thunar_simple_job_new (_thunar_job_load_content_types, 1,
-                                          THUNAR_TYPE_G_FILE_LIST, files);
-
-  g_signal_connect_swapped (job, "finished", G_CALLBACK (thunar_g_list_free_full), copy);
+                                          THUNAR_TYPE_G_FILE_HASH_TABLE, files);
   return job;
 }
+
+
+
+static gboolean
+_thunar_job_load_statusbar_text (ThunarJob *job,
+                                 GArray    *param_values,
+                                 GError   **error)
+{
+  ThunarStandardView *standard_view;
+  ThunarFile         *thunar_folder;
+  GHashTable         *thunar_files;
+  gboolean            show_file_size_binary_format;
+  ThunarDateStyle     date_style;
+  const gchar        *date_custom_style;
+  gchar              *text_for_files;
+  gchar              *temp_string;
+  GList              *text_list = NULL;
+  guint               status_bar_active_info;
+  guint64             size;
+
+  if (exo_job_set_error_if_cancelled (EXO_JOB (job), error))
+    return FALSE;
+
+  standard_view                = g_value_get_object  (&g_array_index (param_values, GValue, 0));
+  thunar_folder                = g_value_get_object  (&g_array_index (param_values, GValue, 1));
+  thunar_files                 = g_value_get_boxed   (&g_array_index (param_values, GValue, 2));
+  show_file_size_binary_format = g_value_get_boolean (&g_array_index (param_values, GValue, 3));
+  date_style                   = g_value_get_enum    (&g_array_index (param_values, GValue, 4));
+  date_custom_style            = g_value_get_string  (&g_array_index (param_values, GValue, 5));
+  status_bar_active_info       = g_value_get_uint    (&g_array_index (param_values, GValue, 6));
+
+  text_for_files = thunar_util_get_statusbar_text_for_files (thunar_files,
+                                                             show_file_size_binary_format,
+                                                             date_style,
+                                                             date_custom_style,
+                                                             status_bar_active_info);
+
+  /* the text is about a selection of files or about a folder */
+  if (thunar_folder == 0)
+    {
+      temp_string = g_strdup_printf (_ ("Selection: %s"), text_for_files);
+      text_list = g_list_append (text_list, temp_string);
+      g_free (text_for_files);
+    }
+  else
+    {
+      text_list = g_list_append (text_list, text_for_files);
+
+      /* check if we can determine the amount of free space for the volume */
+      if (thunar_g_file_get_free_space (thunar_file_get_file (thunar_folder), &size, NULL))
+        {
+          /* humanize the free space */
+          gchar *size_string = g_format_size_full (size, show_file_size_binary_format ? G_FORMAT_SIZE_IEC_UNITS : G_FORMAT_SIZE_DEFAULT);
+          temp_string = g_strdup_printf (_ ("Free space: %s"), size_string);
+          text_list = g_list_append (text_list, temp_string);
+          g_free (size_string);
+        }
+    }
+
+  temp_string = thunar_util_strjoin_list (text_list, "  |  ");
+  g_list_free_full (text_list, g_free);
+
+  thunar_standard_view_set_statusbar_text (standard_view, temp_string);
+  g_free (temp_string);
+
+  return TRUE;
+}
+
+
+
+ThunarJob *
+thunar_io_jobs_load_statusbar_text_for_folder (ThunarStandardView *standard_view,
+                                               ThunarFolder       *folder)
+{
+  ThunarPreferences *preferences;
+  gboolean           show_file_size_binary_format;
+  ThunarDateStyle    date_style;
+  const gchar       *date_custom_style;
+  guint              status_bar_active_info;
+  GHashTable        *files;
+  ThunarFile        *file;
+
+  preferences = thunar_preferences_get ();
+  g_object_get (G_OBJECT (preferences), "misc-date-style", &date_style, 
+                                        "misc_date-custom-style", &date_custom_style, 
+                                        "misc-file-size-binary", &show_file_size_binary_format,
+                                        "misc-status-bar-active-info", &status_bar_active_info, NULL);
+
+  files = thunar_folder_get_files (folder);
+  file = thunar_folder_get_corresponding_file (folder);
+
+  if (file == NULL)
+    return NULL;
+
+  ThunarJob *job = thunar_simple_job_new (_thunar_job_load_statusbar_text, 7,
+                                          THUNAR_TYPE_STANDARD_VIEW, g_object_ref (standard_view),
+                                          THUNAR_TYPE_FILE, g_object_ref (file),
+                                          THUNAR_TYPE_G_FILE_HASH_TABLE, files,
+                                          G_TYPE_BOOLEAN, show_file_size_binary_format,
+                                          THUNAR_TYPE_DATE_STYLE, date_style,
+                                          G_TYPE_STRING, date_custom_style,
+                                          G_TYPE_UINT, status_bar_active_info);
+
+  g_signal_connect_swapped (job, "finished", G_CALLBACK (g_object_unref), file);
+  g_signal_connect_swapped (job, "finished", G_CALLBACK (g_object_unref), standard_view);
+  return job;
+}
+
+
+
+ThunarJob *
+thunar_io_jobs_load_statusbar_text_for_selection (ThunarStandardView *standard_view, GHashTable *selected_files)
+{
+  ThunarPreferences *preferences;
+  gboolean           show_file_size_binary_format;
+  ThunarDateStyle    date_style;
+  const gchar       *date_custom_style;
+  guint              status_bar_active_info;
+
+  preferences = thunar_preferences_get ();
+  g_object_get (G_OBJECT (preferences),"misc-date-style", &date_style, 
+                                       "misc_date-custom-style", &date_custom_style, 
+                                       "misc-file-size-binary", &show_file_size_binary_format,
+                                       "misc-status-bar-active-info", &status_bar_active_info, NULL);
+
+
+  ThunarJob *job = thunar_simple_job_new (_thunar_job_load_statusbar_text, 7,
+                                          THUNAR_TYPE_STANDARD_VIEW, g_object_ref (standard_view),
+                                          THUNAR_TYPE_FILE, NULL,
+                                          THUNAR_TYPE_G_FILE_HASH_TABLE, selected_files,
+                                          G_TYPE_BOOLEAN, show_file_size_binary_format,
+                                          THUNAR_TYPE_DATE_STYLE, date_style,
+                                          G_TYPE_STRING, date_custom_style,
+                                          G_TYPE_UINT, status_bar_active_info);
+
+  g_signal_connect_swapped (job, "finished", G_CALLBACK (g_object_unref), standard_view);
+  return job;
+}
\ No newline at end of file
