@@ -34,7 +34,6 @@
 
 #include "thunar/thunar-application.h"
 #include "thunar/thunar-file.h"
-#include "thunar/thunar-file-monitor.h"
 #include "thunar/thunar-gobject-extensions.h"
 #include "thunar/thunar-list-model.h"
 #include "thunar/thunar-preferences.h"
@@ -135,8 +134,8 @@ static gint               thunar_list_model_cmp_func                    (gconstp
                                                                          gconstpointer                 b,
                                                                          gpointer                      user_data);
 static void               thunar_list_model_sort                        (ThunarListModel              *store);
-static void               thunar_list_model_file_changed                (ThunarFileMonitor            *file_monitor,
-                                                                         ThunarFile                   *file,
+static void               thunar_list_model_files_changed               (ThunarFolder                 *folder,
+                                                                         GHashTable                   *files,
                                                                          ThunarListModel              *store);
 static void               thunar_list_model_folder_destroy              (ThunarFolder                 *folder,
                                                                          ThunarListModel              *store);
@@ -147,13 +146,13 @@ static void               thunar_list_model_notify_loading              (ThunarF
                                                                          GParamSpec                   *pspec,
                                                                          ThunarListModel              *model);
 static void               thunar_list_model_files_added                 (ThunarFolder                 *folder,
-                                                                         GList                        *files,
+                                                                         GHashTable                   *files,
                                                                          ThunarListModel              *store);
 static void               thunar_list_model_files_removed               (ThunarFolder                 *folder,
-                                                                         GList                        *files,
+                                                                         GHashTable                   *files,
                                                                          ThunarListModel              *store);
 static void               thunar_list_model_insert_files                (ThunarListModel              *store,
-                                                                         GList                        *files);
+                                                                         GHashTable                   *files);
 
 static gboolean           thunar_list_model_get_case_sensitive          (ThunarListModel              *store);
 static void               thunar_list_model_set_case_sensitive          (ThunarListModel              *store,
@@ -241,20 +240,14 @@ struct _ThunarListModel
    */
   gchar **search_terms;
 
-  /* Use the shared ThunarFileMonitor instance, so we
-   * do not need to connect "changed" handler to every
-   * file in the model.
-   */
-  ThunarFileMonitor *file_monitor;
-
   /* ids for the "row-inserted" and "row-deleted" signals
    * of GtkTreeModel to speed up folder changing.
    */
   guint          row_inserted_id;
   guint          row_deleted_id;
 
-  gboolean       sort_case_sensitive : 1;
-  gboolean       sort_folders_first : 1;
+  guint          sort_case_sensitive : 1;
+  guint          sort_folders_first : 1;
   gint           sort_sign;   /* 1 = ascending, -1 descending */
   ThunarSortFunc sort_func;
 
@@ -264,7 +257,7 @@ struct _ThunarListModel
    * and adds them in the model. The list is then emptied.
    */
   ThunarJob     *recursive_search_job;
-  GList         *files_to_add;
+  GHashTable    *files_to_add;
   GMutex         mutex_files_to_add;
 
   /* used to stop the periodic call to thunar_list_model_add_search_files when the search is finished/canceled */
@@ -484,16 +477,10 @@ thunar_list_model_init (ThunarListModel *store)
   store->sort_sign = 1;
   store->sort_func = thunar_file_compare_by_name;
   store->rows = g_sequence_new (g_object_unref);
+  store->files_to_add = g_hash_table_new (g_direct_hash, NULL);
   g_mutex_init (&store->mutex_files_to_add);
 
   store->loading = FALSE;
-
-  /* connect to the shared ThunarFileMonitor, so we don't need to
-   * connect "changed" to every single ThunarFile we own.
-   */
-  store->file_monitor = thunar_file_monitor_get_default ();
-  g_signal_connect (G_OBJECT (store->file_monitor), "file-changed",
-                    G_CALLBACK (thunar_list_model_file_changed), store);
 }
 
 
@@ -521,16 +508,12 @@ thunar_list_model_finalize (GObject *object)
       g_source_remove (store->update_search_results_timeout_id);
       store->update_search_results_timeout_id = 0;
     }
-  thunar_g_list_free_full (store->files_to_add);
+  g_hash_table_destroy (store->files_to_add);
   store->files_to_add = NULL;
 
   g_sequence_free (store->rows);
   g_mutex_clear (&store->mutex_files_to_add);
 
-  /* disconnect from the file monitor */
-  g_signal_handlers_disconnect_by_func (G_OBJECT (store->file_monitor), thunar_list_model_file_changed, store);
-  g_object_unref (G_OBJECT (store->file_monitor));
-
   g_free (store->date_custom_style);
 
   g_strfreev (store->search_terms);
@@ -784,7 +767,7 @@ thunar_list_model_get_value (GtkTreeModel *model,
   ThunarFile   *file;
   ThunarFolder *folder;
   gchar        *str;
-  guint32       item_count;
+  gint32        item_count;
   GFile        *g_file;
   GFile        *g_file_parent;
 
@@ -980,7 +963,10 @@ thunar_list_model_get_value (GtkTreeModel *model,
           else if (THUNAR_LIST_MODEL (model)->folder_item_count == THUNAR_FOLDER_ITEM_COUNT_ALWAYS)
             {
               item_count = thunar_file_get_file_count (file, G_CALLBACK (thunar_list_model_file_count_callback), model);
-              g_value_take_string (value, g_strdup_printf (ngettext ("%u item", "%u items", item_count), item_count));
+              if (item_count < 0)
+                g_value_take_string (value, g_strdup (_("unknown")));
+              else
+                g_value_take_string (value, g_strdup_printf (ngettext ("%u item", "%u items", item_count), item_count));
             }
 
           /* If the option is set to always show folder sizes as item counts only for local files,
@@ -990,7 +976,10 @@ thunar_list_model_get_value (GtkTreeModel *model,
               if (thunar_file_is_local (file))
                 {
                   item_count = thunar_file_get_file_count (file, G_CALLBACK (thunar_list_model_file_count_callback), model);
-                  g_value_take_string (value, g_strdup_printf (ngettext ("%u item", "%u items", item_count), item_count));
+                  if (item_count < 0)
+                    g_value_take_string (value, g_strdup (_("unknown")));
+                  else
+                    g_value_take_string (value, g_strdup_printf (ngettext ("%u item", "%u items", item_count), item_count));
                 }
               else
                 g_value_take_string (value, thunar_file_get_size_string_formatted (file, THUNAR_LIST_MODEL (model)->file_size_binary));
@@ -1402,10 +1391,11 @@ thunar_list_model_sort (ThunarListModel *store)
 
 
 static void
-thunar_list_model_file_changed (ThunarFileMonitor *file_monitor,
-                                ThunarFile        *file,
-                                ThunarListModel   *store)
+thunar_list_model_files_changed (ThunarFolder    *folder,
+                                 GHashTable      *files,
+                                 ThunarListModel *store)
 {
+  ThunarFile    *file;
   GSequenceIter *row;
   GSequenceIter *end;
   gint           pos_after;
@@ -1415,108 +1405,110 @@ thunar_list_model_file_changed (ThunarFileMonitor *file_monitor,
   gint           i, j;
   GtkTreePath   *path;
   GtkTreeIter    iter;
-  gboolean       found = FALSE;
-  GList          node;
   GSList        *hidden_link = NULL;
 
-  _thunar_return_if_fail (THUNAR_IS_FILE_MONITOR (file_monitor) || file_monitor == NULL);
   _thunar_return_if_fail (THUNAR_IS_LIST_MODEL (store));
-  _thunar_return_if_fail (THUNAR_IS_FILE (file));
 
   row = g_sequence_get_begin_iter (store->rows);
   end = g_sequence_get_end_iter (store->rows);
-
-  while (row != end)
+  for (;row != end; row = g_sequence_iter_next (row))
     {
-      if (G_UNLIKELY (g_sequence_get (row) == file))
+      /* check if that file was changed */
+      file = g_hash_table_lookup (files, g_sequence_get (row));
+      if (file == NULL)
+        continue;
+        
+      pos_before = g_sequence_iter_get_position (row);
+
+      /* this file is hidden now & show_hidden is FALSE
+      * so we should remove this file from the view and store
+      * it in the hidden list */
+      if (thunar_file_is_hidden (file) && !store->show_hidden)
         {
-          found = TRUE;
+          GHashTable *hidden_files;
 
-          /* this file is hidden now & show_hidden is FALSE
-           * so we should remove this file from the view and store
-           * it in the hidden list */
-          if (thunar_file_is_hidden (file) && !store->show_hidden)
-            {
-              hidden_link = g_slist_find (store->hidden, file);
-              if (hidden_link == NULL)
-                store->hidden = g_slist_prepend (store->hidden, g_object_ref (file));
-              node.data = file;
-              node.next = NULL;
-              node.prev = NULL;
-              thunar_list_model_files_removed (store->folder, &node, store);
-              return;
-            }
-
-          /* generate the iterator for this row */
-          GTK_TREE_ITER_INIT (iter, store->stamp, row);
-
-          _thunar_assert (pos_before == g_sequence_iter_get_position (row));
-
-          /* check if the sorting changed */
-          g_sequence_sort_changed (row, thunar_list_model_cmp_func, store);
-          pos_after = g_sequence_iter_get_position (row);
-          if (pos_after != pos_before)
-            {
-              /* do swap sorting here since its much faster than a complete sort */
-              length = g_sequence_get_length (store->rows);
-              if (G_LIKELY (length < STACK_ALLOC_LIMIT))
-                new_order = g_newa (gint, length);
-              else
-                new_order = g_new (gint, length);
-
-              /* new_order[newpos] = oldpos */
-              for (i = 0, j = 0; i < length; ++i)
-                {
-                  if (G_UNLIKELY (i == pos_after))
-                    {
-                      new_order[i] = pos_before;
-                    }
-                  else
-                    {
-                      if (G_UNLIKELY (j == pos_before))
-                        j++;
-                      new_order[i] = j++;
-                    }
-                }
-
-              /* tell the view about the new item order */
-              path = gtk_tree_path_new_first ();
-              gtk_tree_model_rows_reordered (GTK_TREE_MODEL (store), path, NULL, new_order);
-              gtk_tree_path_free (path);
-
-              /* clean up if we used the heap */
-              if (G_UNLIKELY (length >= STACK_ALLOC_LIMIT))
-                g_free (new_order);
-
-              store->file_was_sorted = TRUE;
-            }
-
-          /* notify the view that it has to redraw the file */
-          path = gtk_tree_path_new_from_indices (pos_before, -1);
-          gtk_tree_model_row_changed (GTK_TREE_MODEL (store), path, &iter);
-          gtk_tree_path_free (path);
-          break;
+          hidden_link = g_slist_find (store->hidden, file);
+          if (hidden_link == NULL)
+            store->hidden = g_slist_prepend (store->hidden, g_object_ref (file));
+          hidden_files = g_hash_table_new (g_direct_hash, NULL);
+          g_hash_table_add (hidden_files, file);
+          thunar_list_model_files_removed (store->folder, hidden_files, store);
+          g_hash_table_destroy (hidden_files);
+          return;
         }
 
-      row = g_sequence_iter_next (row);
-      pos_before++;
-    }
+      /* generate the iterator for this row */
+      GTK_TREE_ITER_INIT (iter, store->stamp, row);
+      
+      /* check if the sorting changed */
+      g_sequence_sort_changed (row, thunar_list_model_cmp_func, store);
+      pos_after = g_sequence_iter_get_position (row);
+
+      /* if 'g_sequence_sort_changed' changed the sorting, the positions will differ now */
+      if (pos_after != pos_before)
+        {
+          /* do swap sorting here since its much faster than a complete sort */
+          length = g_sequence_get_length (store->rows);
+          if (G_LIKELY (length < STACK_ALLOC_LIMIT))
+            new_order = g_newa (gint, length);
+          else
+            new_order = g_new (gint, length);
+
+          /* new_order[newpos] = oldpos */
+          for (i = 0, j = 0; i < length; ++i)
+            {
+              if (G_UNLIKELY (i == pos_after))
+                {
+                  new_order[i] = pos_before;
+                }
+              else
+                {
+                  if (G_UNLIKELY (j == pos_before))
+                    j++;
+                  new_order[i] = j++;
+                }
+            }
+
+          /* tell the view about the new item order */
+          path = gtk_tree_path_new_first ();
+          gtk_tree_model_rows_reordered (GTK_TREE_MODEL (store), path, NULL, new_order);
+          gtk_tree_path_free (path);
+
+          /* clean up if we used the heap */
+          if (G_UNLIKELY (length >= STACK_ALLOC_LIMIT))
+            g_free (new_order);
+
+          store->file_was_sorted = TRUE;
+        }
+      else
+        {
+          /* just notify the view that it has to redraw the file */
+          path = gtk_tree_path_new_from_indices (pos_before, -1);
+          gtk_tree_model_row_changed (GTK_TREE_MODEL (store), path, &iter);//this line 
+          gtk_tree_path_free (path);
+        }
+
+   } /* end while loop (for each row) */
 
-  if (found)
-    return;
 
   /* maybe this file was a hidden file but now it's not
-   * in such a case we need to emit a "files-added" for this file
-   * and remove it from the hidden list */
-  hidden_link = g_slist_find (store->hidden, file);
-  if (hidden_link == NULL || thunar_file_is_hidden (file))
-    return;
-  g_object_unref (hidden_link->data);
-  store->hidden = g_slist_delete_link (store->hidden, hidden_link);
-  node.data = file;
-  node.next = NULL;
-  node.prev = NULL;
-  thunar_list_model_files_added (store->folder, &node, store);
+  * in such a case we need to emit a "files-added" for this file
+  * and remove it from the hidden list */
+  for (GSList *lp = store->hidden; lp != NULL; lp = g_slist_next (lp))
+    {
+      GHashTable *hidden_files;
+
+      hidden_link = g_hash_table_lookup (files, lp->data);
+      if (hidden_link == NULL || thunar_file_is_hidden (THUNAR_FILE (hidden_link)))
+        continue;
+
+      g_object_unref (hidden_link->data);
+      store->hidden = g_slist_delete_link (store->hidden, hidden_link);
+      hidden_files = g_hash_table_new (g_direct_hash, NULL);
+      g_hash_table_add (hidden_files, hidden_link);
+      thunar_list_model_files_added (store->folder, hidden_files, store);
+      g_hash_table_destroy (hidden_files);
+    }
 }
 
 
@@ -1569,14 +1561,15 @@ thunar_list_model_notify_loading (ThunarFolder    *folder,
 
 static void
 thunar_list_model_files_added (ThunarFolder    *folder,
-                               GList           *files,
+                               GHashTable      *files,
                                ThunarListModel *store)
 {
-  GList       *filtered;
-  GList       *lp;
-  ThunarFile  *file;
-  gboolean     matched;
-  gchar       *name_n;
+  GHashTable    *filtered;
+  ThunarFile    *file;
+  gboolean       matched;
+  gchar         *name_n;
+  gpointer       key;
+  GHashTableIter iter;
 
   /* pass the list directly if not currently showing search results */
   if (store->search_terms == NULL)
@@ -1586,40 +1579,50 @@ thunar_list_model_files_added (ThunarFolder    *folder,
     }
 
   /* otherwise, filter out files that don't match the current search terms */
-  filtered = NULL;
-  for (lp = files; lp != NULL; lp = lp->next)
+  filtered = g_hash_table_new (g_direct_hash, NULL);
+
+  g_hash_table_iter_init (&iter, files);
+  while (g_hash_table_iter_next (&iter, &key, NULL))
     {
-      /* take a reference on that file */
-      file = THUNAR_FILE (g_object_ref (G_OBJECT (lp->data)));
-      _thunar_return_if_fail (THUNAR_IS_FILE (file));
+      file = THUNAR_FILE (key);
+      if (THUNAR_IS_FILE (file) == FALSE)
+        {
+          g_warning ("failed to add file to search results");
+          continue;
+        }
 
       name_n = (gchar *)thunar_file_get_display_name (file);
+      if (name_n == NULL)
+      {
+          g_warning ("failed to get display name");
+          continue;
+      }
+
       name_n = thunar_g_utf8_normalize_for_search (name_n, TRUE, TRUE);
       matched = thunar_util_search_terms_match (store->search_terms, name_n);
       g_free (name_n);
 
-      if (! matched)
-        g_object_unref (file);
-      else
-        filtered = g_list_append (filtered, file);
+      if (matched)
+        g_hash_table_add (filtered, file);
     }
   thunar_list_model_insert_files (store, filtered);
-  thunar_g_list_free_full (filtered);
+  g_hash_table_destroy (filtered);
 }
 
 
 static void
 thunar_list_model_insert_files (ThunarListModel *store,
-                                GList           *files)
+                                GHashTable      *files)
 {
   GtkTreePath   *path;
   GtkTreeIter    iter;
   ThunarFile    *file;
   gint          *indices;
   GSequenceIter *row;
-  GList         *lp;
   gboolean       has_handler;
   gboolean       search_mode;
+  gpointer       key;
+  GHashTableIter file_iter;
 
   /* we use a simple trick here to avoid allocating
    * GtkTreePath's again and again, by simply accessing
@@ -1634,10 +1637,11 @@ thunar_list_model_insert_files (ThunarListModel *store,
 
   /* process all added files */
   search_mode = (store->search_terms != NULL);
-  for (lp = files; lp != NULL; lp = lp->next)
+  g_hash_table_iter_init (&file_iter, files);
+  while (g_hash_table_iter_next (&file_iter, &key, NULL))
     {
       /* take a reference on that file */
-      file = THUNAR_FILE (g_object_ref (G_OBJECT (lp->data)));
+      file = THUNAR_FILE (g_object_ref (G_OBJECT (key)));
       _thunar_return_if_fail (THUNAR_IS_FILE (file));
 
       /* check if the file should be stashed in the hidden list */
@@ -1679,21 +1683,26 @@ thunar_list_model_insert_files (ThunarListModel *store,
 
 static void
 thunar_list_model_files_removed (ThunarFolder    *folder,
-                                 GList           *files,
+                                 GHashTable      *files,
                                  ThunarListModel *store)
 {
-  GList         *lp;
   GSequenceIter *row;
   GSequenceIter *end;
   GSequenceIter *next;
   GtkTreePath   *path;
   gboolean       found;
   gboolean       search_mode;
+  gpointer       key;
+  ThunarFile    *file;
+  GHashTableIter iter;
 
   /* drop all the referenced files from the model */
   search_mode = (store->search_terms != NULL);
-  for (lp = files; lp != NULL; lp = lp->next)
+  g_hash_table_iter_init (&iter, files);
+  while (g_hash_table_iter_next (&iter, &key, NULL))
     {
+      file = THUNAR_FILE (G_OBJECT (key));
+
       row = g_sequence_get_begin_iter (store->rows);
       end = g_sequence_get_end_iter (store->rows);
 
@@ -1703,7 +1712,7 @@ thunar_list_model_files_removed (ThunarFolder    *folder,
         {
           next = g_sequence_iter_next (row);
 
-          if (g_sequence_get (row) == lp->data)
+          if (g_sequence_get (row) == file)
             {
               /* indicate that file was removed from this model */
               store->file_was_removed = TRUE;
@@ -1734,9 +1743,9 @@ thunar_list_model_files_removed (ThunarFolder    *folder,
             {
               /* file is hidden */
               /* this only makes sense when not storing search results */
-              _thunar_assert (g_slist_find (store->hidden, lp->data) != NULL);
-              store->hidden = g_slist_remove (store->hidden, lp->data);
-              g_object_unref (G_OBJECT (lp->data));
+              _thunar_assert (g_slist_find (store->hidden, file) != NULL);
+              store->hidden = g_slist_remove (store->hidden, file);
+              g_object_unref (G_OBJECT (file));
             }
         }
     }
@@ -1956,8 +1965,9 @@ thunar_list_model_add_search_files (ThunarStandardViewModel *model,
   ThunarListModel *_model = THUNAR_LIST_MODEL (model);
 
   g_mutex_lock (&_model->mutex_files_to_add);
-
-  _model->files_to_add = g_list_concat (_model->files_to_add, files);
+  
+  for (GList *lp = files; lp != NULL; lp = g_list_next (lp))
+    g_hash_table_add (_model->files_to_add, lp->data);
 
   g_mutex_unlock (&_model->mutex_files_to_add);
 }
@@ -1972,8 +1982,7 @@ thunar_list_model_update_search_files (ThunarListModel *model)
   if (model->files_to_add != NULL)
     {
       thunar_list_model_insert_files (model, model->files_to_add);
-      g_list_free (model->files_to_add);
-      model->files_to_add = NULL;
+      g_hash_table_remove_all (model->files_to_add);
     }
 
   g_mutex_unlock (&model->mutex_files_to_add);
@@ -1999,6 +2008,7 @@ thunar_list_model_cancel_search_job (ThunarListModel *model)
 
 
 
+G_NORETURN
 static void
 thunar_list_model_search_error (ThunarJob *job)
 {
@@ -2025,8 +2035,7 @@ thunar_list_model_search_finished (ThunarJob       *job,
       store->update_search_results_timeout_id = 0;
     }
 
-  thunar_g_list_free_full (store->files_to_add);
-  store->files_to_add = NULL;
+  g_hash_table_remove_all (store->files_to_add);
 
   g_signal_emit_by_name (store, "search-done");
 }
@@ -2064,7 +2073,7 @@ thunar_list_model_set_folder (ThunarStandardViewModel *model,
   ThunarListModel   *store = THUNAR_LIST_MODEL (model);
   GtkTreePath   *path;
   gboolean       has_handler;
-  GList         *files;
+  GHashTable    *files;
   GSequenceIter *row;
   GSequenceIter *end;
   GSequenceIter *next;
@@ -2082,8 +2091,7 @@ thunar_list_model_set_folder (ThunarStandardViewModel *model,
           g_source_remove (store->update_search_results_timeout_id);
           store->update_search_results_timeout_id = 0;
         }
-      thunar_g_list_free_full (store->files_to_add);
-      store->files_to_add = NULL;
+      g_hash_table_remove_all (store->files_to_add);
 
       /* check if we have any handlers connected for "row-deleted" */
       has_handler = g_signal_has_handler_pending (G_OBJECT (store), store->row_deleted_id, 0, FALSE);
@@ -2180,21 +2188,19 @@ thunar_list_model_set_folder (ThunarStandardViewModel *model,
 
       /* insert the files */
       if (files != NULL)
-        {
-          thunar_list_model_insert_files (store, files);
-          g_list_free (files);
-        }
+        thunar_list_model_insert_files (store, files);
 
       /* connect signals to the new folder */
       g_signal_connect (G_OBJECT (store->folder), "destroy", G_CALLBACK (thunar_list_model_folder_destroy), store);
       g_signal_connect (G_OBJECT (store->folder), "error", G_CALLBACK (thunar_list_model_folder_error), store);
       g_signal_connect (G_OBJECT (store->folder), "files-added", G_CALLBACK (thunar_list_model_files_added), store);
       g_signal_connect (G_OBJECT (store->folder), "files-removed", G_CALLBACK (thunar_list_model_files_removed), store);
+      g_signal_connect (G_OBJECT (store->folder), "files-changed", G_CALLBACK (thunar_list_model_files_changed), store);
       g_signal_connect (G_OBJECT (store->folder), "notify::loading", G_CALLBACK (thunar_list_model_notify_loading), store);
 
-      /* notify for "loading" if already loaded */
+      /* If the folder is already loaded, directly update the loading state */
       if (!thunar_folder_get_loading (store->folder))
-          g_object_notify (G_OBJECT (store->folder), "loading");
+        thunar_list_model_notify_loading (store->folder, NULL, store);
     }
 
   /* notify listeners that we have a new folder */
@@ -2604,8 +2610,11 @@ static void
 thunar_list_model_file_count_callback (ExoJob  *job,
                                        gpointer model)
 {
+  ThunarListModel *store = THUNAR_LIST_MODEL (model);
+
   GArray     *param_values;
   ThunarFile *file;
+  GHashTable *files;
 
   if (job == NULL)
     return;
@@ -2616,5 +2625,8 @@ thunar_list_model_file_count_callback (ExoJob  *job,
   if (file == NULL)
     return;
 
-  thunar_list_model_file_changed (NULL, file, THUNAR_LIST_MODEL (model));
+  files = g_hash_table_new (g_direct_hash, NULL);
+  g_hash_table_add (files, file);
+  thunar_list_model_files_changed (store->folder, files, store);
+  g_hash_table_destroy (files);
 }
