@@ -18,11 +18,13 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
-#include <thunar/thunar-navigator.h>
-#include <thunar/thunar-private.h>
+#include <libxfce4util/libxfce4util.h>
+
+#include "thunar/thunar-navigator.h"
+#include "thunar/thunar-private.h"
 
 
 
