@@ -43,6 +43,7 @@
 #include "thunar/thunar-icon-renderer.h"
 #include "thunar/thunar-list-model.h"
 #include "thunar/thunar-path-entry.h"
+#include "thunar/thunar-preferences.h"
 #include "thunar/thunar-private.h"
 #include "thunar/thunar-util.h"
 #include "thunar/thunar-window.h"
@@ -471,7 +472,8 @@ thunar_path_entry_motion_notify_event (GtkWidget      *widget,
       icon = thunar_icon_factory_load_file_icon (path_entry->icon_factory,
                                                  path_entry->current_file,
                                                  THUNAR_FILE_ICON_STATE_DEFAULT,
-                                                 MAX (size, 16), scale_factor);
+                                                 MAX (size, 16), scale_factor,
+                                                 FALSE, NULL);
       if (G_LIKELY (icon != NULL))
         {
           surface = gdk_cairo_surface_create_from_pixbuf (icon, scale_factor, gtk_widget_get_window (widget));
@@ -721,16 +723,21 @@ thunar_path_entry_changed (GtkEditable *editable)
 static void
 thunar_path_entry_update_icon (ThunarPathEntry *path_entry)
 {
-  GdkPixbuf          *icon = NULL;
-  GtkIconTheme       *icon_theme;
-  gint                icon_size;
-  gint                scale_factor;
+  ThunarPreferences *preferences;
+  GdkPixbuf         *icon = NULL;
+  GtkIconTheme      *icon_theme;
+  gint               icon_size;
+  gint               scale_factor;
+  gboolean           use_symbolic_icons;
+
+  preferences = thunar_preferences_get ();
+  g_object_get (G_OBJECT (preferences), "misc-symbolic-icons-in-toolbar", &use_symbolic_icons, NULL);
+  g_object_unref (G_OBJECT (preferences));
 
   if (path_entry->search_mode == TRUE)
     {
-      gtk_entry_set_icon_from_icon_name (GTK_ENTRY (path_entry),
-                                         GTK_ENTRY_ICON_PRIMARY,
-                                         "system-search");
+      gtk_entry_set_icon_from_icon_name (GTK_ENTRY (path_entry), GTK_ENTRY_ICON_PRIMARY,
+                                         use_symbolic_icons ? "system-search-symbolic" : "system-search");
       return;
     }
 
@@ -748,14 +755,16 @@ thunar_path_entry_update_icon (ThunarPathEntry *path_entry)
       icon = thunar_icon_factory_load_file_icon (path_entry->icon_factory,
                                                  path_entry->current_file,
                                                  THUNAR_FILE_ICON_STATE_DEFAULT,
-                                                 icon_size, scale_factor);
+                                                 icon_size, scale_factor,
+                                                 FALSE, NULL);
     }
   else if (G_LIKELY (path_entry->current_folder != NULL))
     {
       icon = thunar_icon_factory_load_file_icon (path_entry->icon_factory,
                                                  path_entry->current_folder,
                                                  THUNAR_FILE_ICON_STATE_DEFAULT,
-                                                 icon_size, scale_factor);
+                                                 icon_size, scale_factor,
+                                                 FALSE, NULL);
     }
 
   if (icon != NULL)
@@ -767,9 +776,8 @@ thunar_path_entry_update_icon (ThunarPathEntry *path_entry)
     }
   else
     {
-      gtk_entry_set_icon_from_icon_name (GTK_ENTRY (path_entry),
-                                         GTK_ENTRY_ICON_PRIMARY,
-                                         "dialog-error-symbolic");
+      gtk_entry_set_icon_from_icon_name (GTK_ENTRY (path_entry), GTK_ENTRY_ICON_PRIMARY,
+                                         use_symbolic_icons ? "dialog-error-symbolic" : "dialog-error");
     }
 }
 
