@@ -83,7 +83,8 @@ enum
   PROP_LAST_RENAMER_DIALOG_WIDTH,
   PROP_LAST_RENAMER_DIALOG_FULLSCREEN,
   PROP_LAST_TOOLBAR_VISIBLE_BUTTONS,
-  PROP_LAST_TOOLBAR_BUTTON_ORDER,
+  PROP_LAST_TOOLBAR_ITEM_ORDER,
+  PROP_LAST_TOOLBAR_ITEMS,
   PROP_MISC_DIRECTORY_SPECIFIC_SETTINGS,
   PROP_MISC_ALWAYS_SHOW_TABS,
   PROP_MISC_VOLUME_MANAGEMENT,
@@ -124,16 +125,21 @@ enum
   PROP_SHORTCUTS_ICON_SIZE,
   PROP_TREE_ICON_EMBLEMS,
   PROP_TREE_ICON_SIZE,
+  PROP_TREE_LINES,
   PROP_MISC_SWITCH_TO_NEW_TAB,
   PROP_MISC_VERTICAL_SPLIT_PANE,
-  PROP_MISC_OPEN_NEW_WINDOWS_IN_SPLIT_VIEW,
+  PROP_MISC_OPEN_NEW_WINDOWS_IN_SPLIT_VIEW, // Drop for or after 4.22
+  PROP_MISC_ALWAYS_ENABLE_SPLIT_VIEW,
   PROP_MISC_COMPACT_VIEW_MAX_CHARS,
   PROP_MISC_HIGHLIGHTING_ENABLED,
   PROP_MISC_UNDO_REDO_HISTORY_SIZE,
   PROP_MISC_CONFIRM_MOVE_TO_TRASH,
   PROP_MISC_MAX_NUMBER_OF_TEMPLATES,
-  PROP_SHOW_LAUNCHER_NAMES_INSTEAD_REAL_FILENAMES,
+  PROP_MISC_DISPLAY_LAUNCHER_NAME_AS_FILENAME,
   PROP_MISC_EXPANDABLE_FOLDERS,
+  PROP_MISC_SYMBOLIC_ICONS_IN_TOOLBAR,
+  PROP_MISC_SYMBOLIC_ICONS_IN_SIDEPANE,
+  PROP_MISC_CTRL_SCROLL_WHEEL_TO_ZOOM,
   N_PROPERTIES,
 };
 
@@ -438,7 +444,7 @@ thunar_preferences_class_init (ThunarPreferencesClass *klass)
       g_param_spec_int ("last-splitview-separator-position",
                         "LastSplitviewSeparatorPosition",
                         NULL,
-                        0, G_MAXINT, 0,
+                        -1, G_MAXINT, -1,
                         EXO_PARAM_READWRITE);
 
   /**
@@ -457,26 +463,31 @@ thunar_preferences_class_init (ThunarPreferencesClass *klass)
    * ThunarPreferences:last-toolbar-visible-buttons:
    *
    * The comma separated list that specifies the visibility of toolbar items.
-   * The order of the default value corresponds to the order in which the buttons are added inside 'thunar_window_location_toolbar_create'
+   * The order of the default value corresponds to the order in which the buttons
+   * are added inside 'thunar_window_location_toolbar_create'.
+   *
+   * This property is only read during the migration to "last-toolbar-items".
    **/
     preferences_props[PROP_LAST_TOOLBAR_VISIBLE_BUTTONS] =
         g_param_spec_string ("last-toolbar-visible-buttons",
                              "LastToolbarVisibleButtons",
                              NULL,
-                             "0,1,1,1,1,0,0,0,0,0,0,0,0,0,1,0,1",
-                             EXO_PARAM_READWRITE);
+                             NULL,
+                             EXO_PARAM_READABLE);
 
   /**
-   * ThunarPreferences:last-toolbar-button-order:
+   * ThunarPreferences:last-toolbar-item-order:
    *
    * The comma separated list that specifies the order of toolbar items.
+   *
+   * This property is only read during the migration to "last-toolbar-items".
    **/
-    preferences_props[PROP_LAST_TOOLBAR_BUTTON_ORDER] =
+    preferences_props[PROP_LAST_TOOLBAR_ITEM_ORDER] =
         g_param_spec_string ("last-toolbar-item-order",
-                             "LastToolbarButtonOrder",
+                             "LastToolbarItemOrder",
                              NULL,
-                             "0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16",
-                             EXO_PARAM_READWRITE);
+                             NULL,
+                             EXO_PARAM_READABLE);
 
   /**
    * ThunarPreferences:misc-directory-specific-settings:
@@ -643,6 +654,22 @@ thunar_preferences_class_init (ThunarPreferencesClass *klass)
                             FALSE,
                             EXO_PARAM_READWRITE);
 
+  /**
+   * ThunarPreferences:last-toolbar-items:
+   *
+   * The comma separated list that specifies the order and visibility of toolbar items.
+   **/
+    preferences_props[PROP_LAST_TOOLBAR_ITEMS] =
+        g_param_spec_string ("last-toolbar-items",
+                             "LastToolbarItems",
+                             NULL,
+                             "view-menubar:0,back:1,forward:1,open-parent:1,open-home:1,"
+                             "new-tab:0,new-window:0,toggle-split-view:0,"
+                             "undo:0,redo:0,zoom-out:0,zoom-in:0,zoom-reset:0,"
+                             "view-as-icons:0,view-as-detailed-list:0,view-as-compact-list:0,"
+                             "location-bar:1,reload:0,search:1",
+                             EXO_PARAM_READWRITE);
+
   /**
    * ThunarPreferences:misc-always-show-tabs:
    *
@@ -1159,6 +1186,18 @@ thunar_preferences_class_init (ThunarPreferencesClass *klass)
                          EXO_PARAM_READWRITE);
 
   /**
+   * ThunarPreferences:misc-tree-lines-in-tree-sidepane:
+   *
+   * Whether to show tree lines or not in the tree side pane.
+   **/
+  preferences_props[PROP_TREE_LINES] =
+      g_param_spec_boolean ("misc-tree-lines-in-tree-sidepane",
+                            "TreeLines",
+                            NULL,
+                            FALSE,
+                            EXO_PARAM_READWRITE);
+
+ /**
  * ThunarPreferences:misc-switch-to-new-tab:
  *
  * Whether to switch to the new tab after opening a folder in a new tab.
@@ -1188,6 +1227,9 @@ thunar_preferences_class_init (ThunarPreferencesClass *klass)
   /**
    * ThunarPreferences:misc-open-new-windows-in-split-view:
    *
+   * DEPRECIATED for 4.20. Should be removed for 4.22 or later versions, together with
+   * the migration code in "thunar_application_open_window()"
+   *
    * If true, all thunar windows will have split view enabled.
    **/
   preferences_props[PROP_MISC_OPEN_NEW_WINDOWS_IN_SPLIT_VIEW] =
@@ -1197,6 +1239,18 @@ thunar_preferences_class_init (ThunarPreferencesClass *klass)
                           FALSE,
                           EXO_PARAM_READWRITE);
 
+  /**
+   * ThunarPreferences:misc-always-enable-split-view:
+   *
+   * If true, all thunar windows will have split view enabled.
+   **/
+  preferences_props[PROP_MISC_ALWAYS_ENABLE_SPLIT_VIEW] =
+    g_param_spec_boolean ("misc-always-enable-split-view",
+                          "MiscAlwaysEnableSplitView",
+                          NULL,
+                          FALSE,
+                          EXO_PARAM_READWRITE);
+
   /**
    * ThunarPreferences:compact-view-max-chars:
    *
@@ -1276,17 +1330,53 @@ thunar_preferences_class_init (ThunarPreferencesClass *klass)
                             EXO_PARAM_READWRITE);
 
   /**
-   * ThunarPreferences:show-launcher-names-instead-real-filenames:
+   * ThunarPreferences:misc-display-launcher-name-as-filename:
    *
-   * Wheter to show real file names or launcher names for .desktop files
+   * Whether to show real file names or launcher names for .desktop files
    **/
-  preferences_props[PROP_SHOW_LAUNCHER_NAMES_INSTEAD_REAL_FILENAMES] =
-       g_param_spec_boolean ("show-launcher-names-instead-real-filenames",
-                             "ShowLauncherNamesInsteadRealFilenames",
+  preferences_props[PROP_MISC_DISPLAY_LAUNCHER_NAME_AS_FILENAME] =
+       g_param_spec_boolean ("misc-display-launcher-name-as-filename",
+                             "MiscDisplayLauncherNameAsFilename",
                              NULL,
                              TRUE,
                              EXO_PARAM_READWRITE);
 
+   /**
+   * ThunarPreferences:misc-symbolic-icons-in-toolbar:
+   *
+   * If true symbolic icons should be used in the toolbar
+   **/
+  preferences_props[PROP_MISC_SYMBOLIC_ICONS_IN_TOOLBAR] =
+      g_param_spec_boolean ("misc-symbolic-icons-in-toolbar",
+                            "MiscSymbolicIconsInToolbar",
+                            NULL,
+                            TRUE,
+                            EXO_PARAM_READWRITE);
+
+   /**
+   * ThunarPreferences:misc-symbolic-icons-in-sidepane:
+   *
+   * If true symbolic icons should be used in the sidepane
+   **/
+  preferences_props[PROP_MISC_SYMBOLIC_ICONS_IN_SIDEPANE] =
+      g_param_spec_boolean ("misc-symbolic-icons-in-sidepane",
+                            "MiscSymbolicIconsInSidepane",
+                            NULL,
+                            FALSE,
+                            EXO_PARAM_READWRITE);
+
+   /**
+    * ThunarPreferences:misc-ctrl-scroll-wheel-to-zoom:
+    *
+    * If true, Ctrl + Scroll Wheel to Zoom is enabled
+    **/
+  preferences_props[PROP_MISC_CTRL_SCROLL_WHEEL_TO_ZOOM] =
+      g_param_spec_boolean ("misc-ctrl-scroll-wheel-to-zoom",
+                            "MiscCtrlScrollWheelToZoom",
+                            NULL,
+                            TRUE,
+                            EXO_PARAM_READWRITE);
+
   /* install all properties */
   g_object_class_install_properties (gobject_class, N_PROPERTIES, preferences_props);
 }
@@ -1368,7 +1458,7 @@ thunar_preferences_get_property (GObject    *object,
       if (G_VALUE_TYPE (value) == G_VALUE_TYPE (&src))
         g_value_copy (&src, value);
       else if (!g_value_transform (&src, value))
-        g_printerr ("Thunar: Failed to transform property %s\n", prop_name);
+        g_warning ("Thunar: Failed to transform property %s", prop_name);
       g_value_unset (&src);
     }
   else
@@ -1430,6 +1520,25 @@ thunar_preferences_set_property (GObject      *object,
 
 
 
+/**
+ * thunar_preferences_has_property:
+ * @preferences : the #ThunarPreferences instance.
+ * @prop_name   : a property name.
+ *
+ * Return value: %TRUE if @prop_name exists, %FALSE otherwise.
+ **/
+gboolean
+thunar_preferences_has_property (ThunarPreferences *preferences,
+                                 const gchar       *prop_name)
+{
+  if (G_UNLIKELY (preferences->channel == NULL))
+    return FALSE;
+
+  return xfconf_channel_has_property (preferences->channel, prop_name);
+}
+
+
+
 static void
 thunar_preferences_prop_changed (XfconfChannel     *channel,
                                  const gchar       *prop_name,
