@@ -155,6 +155,7 @@ struct _ThunarPropertiesDialog
   GtkWidget              *kind_label;
   GtkWidget              *openwith_chooser;
   GtkWidget              *link_label;
+  GtkWidget              *link_label_text;
   GtkWidget              *location_label;
   GtkWidget              *origin_label;
   GtkWidget              *created_label;
@@ -178,9 +179,11 @@ struct _ThunarPropertiesDialog
    * btns under highlights tab */
   GtkWidget              *highlight_buttons;
   GtkWidget              *highlighting_spinner;
-  ThunarJob              *highlight_change_job;
   gulong                  highlight_change_job_finish_signal;
 
+  ThunarJob              *highlight_change_job;
+  ThunarJob              *rename_job;
+
   GtkWidget              *highlight_apply_button;
   GtkWidget              *editor_button;
 
@@ -293,6 +296,8 @@ thunar_properties_dialog_init (ThunarPropertiesDialog *dialog)
 
   dialog->provider_factory = thunarx_provider_factory_get_default ();
 
+
+  dialog->rename_job = NULL;
   dialog->highlight_change_job = NULL;
   dialog->highlight_change_job_finish_signal = 0;
 }
@@ -458,6 +463,7 @@ thunar_properties_dialog_constructed (GObject *object)
   ++row;
 
   label = gtk_label_new (_("Link Target:"));
+  dialog->link_label_text = label;
   gtk_label_set_attributes (GTK_LABEL (label), thunar_pango_attr_list_bold ());
   gtk_label_set_xalign (GTK_LABEL (label), 1.0f);
   gtk_grid_attach (GTK_GRID (grid), label, 0, row, 1, 1);
@@ -953,6 +959,13 @@ thunar_properties_dialog_finalize (GObject *object)
   if (dialog->highlight_change_job != NULL && dialog->highlight_change_job_finish_signal != 0)
     g_signal_handler_disconnect (dialog->highlight_change_job, dialog->highlight_change_job_finish_signal);
 
+  if (dialog->rename_job != NULL)
+    {
+      g_signal_handlers_disconnect_by_data (dialog->rename_job, dialog);
+      g_object_unref (dialog->rename_job);
+      dialog->rename_job = NULL;
+    }
+
   (*G_OBJECT_CLASS (thunar_properties_dialog_parent_class)->finalize) (object);
 }
 
@@ -1098,8 +1111,12 @@ thunar_properties_dialog_rename_finished (ExoJob                 *job,
   _thunar_return_if_fail (THUNAR_IS_PROPERTIES_DIALOG (dialog));
   _thunar_return_if_fail (g_list_length (dialog->files) == 1);
 
-  g_signal_handlers_disconnect_by_data (job, dialog);
-  g_object_unref (job);
+  if (dialog->rename_job != NULL)
+    {
+      g_signal_handlers_disconnect_by_data (job, dialog);
+      g_object_unref (dialog->rename_job);
+      dialog->rename_job = NULL;
+    }
 }
 
 
@@ -1110,7 +1127,6 @@ thunar_properties_dialog_name_activate (GtkWidget              *entry,
 {
   const gchar *old_name;
   const gchar *new_name;
-  ThunarJob   *job;
   ThunarFile  *file;
 
   _thunar_return_if_fail (THUNAR_IS_PROPERTIES_DIALOG (dialog));
@@ -1126,12 +1142,13 @@ thunar_properties_dialog_name_activate (GtkWidget              *entry,
   old_name = thunar_file_get_basename (file);
   if (g_utf8_collate (new_name, old_name) != 0)
     {
-      job = thunar_io_jobs_rename_file (file, new_name, THUNAR_OPERATION_LOG_OPERATIONS);
-      exo_job_launch (EXO_JOB (job));
-      if (job != NULL)
+      dialog->rename_job = thunar_io_jobs_rename_file (file, new_name, THUNAR_OPERATION_LOG_OPERATIONS);
+
+      if (dialog->rename_job != NULL)
         {
-          g_signal_connect (job, "error", G_CALLBACK (thunar_properties_dialog_rename_error), dialog);
-          g_signal_connect (job, "finished", G_CALLBACK (thunar_properties_dialog_rename_finished), dialog);
+          exo_job_launch (EXO_JOB (dialog->rename_job));
+          g_signal_connect (dialog->rename_job, "error", G_CALLBACK (thunar_properties_dialog_rename_error), dialog);
+          g_signal_connect (dialog->rename_job, "finished", G_CALLBACK (thunar_properties_dialog_rename_finished), dialog);
         }
     }
 }
@@ -1386,6 +1403,7 @@ thunar_properties_dialog_update_single (ThunarPropertiesDialog *dialog)
     {
       display_name = g_filename_display_name (path);
       gtk_label_set_text (GTK_LABEL (dialog->link_label), display_name);
+      gtk_widget_set_tooltip_text (dialog->link_label, display_name);
       gtk_widget_show (dialog->link_label);
       g_free (display_name);
     }
@@ -1414,6 +1432,7 @@ thunar_properties_dialog_update_single (ThunarPropertiesDialog *dialog)
     {
       display_name = g_file_get_parse_name (thunar_file_get_file (parent_file));
       gtk_label_set_text (GTK_LABEL (dialog->location_label), display_name);
+      gtk_widget_set_tooltip_text (dialog->location_label, display_name);
       gtk_widget_show (dialog->location_label);
       g_object_unref (G_OBJECT (parent_file));
       g_free (display_name);
@@ -1572,6 +1591,8 @@ thunar_properties_dialog_update_multiple (ThunarPropertiesDialog *dialog)
   ThunarFile  *parent_file = NULL;
   ThunarFile  *tmp_parent;
   gboolean     has_trashed_files = FALSE;
+  GString     *str_of_resolved_paths = g_string_new(NULL);
+  const gchar *resolved_path;
 
   _thunar_return_if_fail (THUNAR_IS_PROPERTIES_DIALOG (dialog));
   _thunar_return_if_fail (g_list_length (dialog->files) > 1);
@@ -1598,6 +1619,23 @@ thunar_properties_dialog_update_multiple (ThunarPropertiesDialog *dialog)
       _thunar_assert (THUNAR_IS_FILE (lp->data));
       file = THUNAR_FILE (lp->data);
 
+      resolved_path = thunar_file_is_symlink (file) ? thunar_file_get_symlink_target (file) : NULL;
+      /* check if the file is a symlink, and get its resolved path */
+      if (resolved_path != NULL)
+        {
+          /* If there is even a single symlink then make 'Link Targets' visible and rename it to 'Link Targets' */
+          gtk_widget_show (dialog->link_label);
+          gtk_label_set_text (GTK_LABEL (dialog->link_label_text), _("Link Targets:"));
+
+          /* Add , only if there was a resolved path before*/
+          if (str_of_resolved_paths->len != 0)
+              g_string_append (str_of_resolved_paths, ", ");
+
+          g_string_append (str_of_resolved_paths, thunar_file_get_basename (file));
+          g_string_append (str_of_resolved_paths, ": ");
+          g_string_append (str_of_resolved_paths, resolved_path);
+        }
+
       /* append the name */
       if (!first_file)
         g_string_append (names_string, ", ");
@@ -1682,11 +1720,25 @@ thunar_properties_dialog_update_multiple (ThunarPropertiesDialog *dialog)
       gtk_label_set_text (GTK_LABEL (dialog->kind_label), _("mixed"));
     }
 
+  /* update the link target */
+  if (G_LIKELY (str_of_resolved_paths->len > 0))
+    {
+      gtk_label_set_text (GTK_LABEL (dialog->link_label), str_of_resolved_paths->str);
+      gtk_widget_set_tooltip_text (dialog->link_label, str_of_resolved_paths->str);
+      gtk_widget_show (dialog->link_label);
+    }
+  else
+    {
+      gtk_widget_hide (dialog->link_label);
+    }
+  g_string_free (str_of_resolved_paths, TRUE);
+
   /* update the file or folder location (parent) */
   if (G_UNLIKELY (parent_file != NULL))
     {
       display_name = g_file_get_parse_name (thunar_file_get_file (parent_file));
       gtk_label_set_text (GTK_LABEL (dialog->location_label), display_name);
+      gtk_widget_set_tooltip_text (dialog->location_label, display_name);
       gtk_widget_show (dialog->location_label);
       g_object_unref (G_OBJECT (parent_file));
       g_free (display_name);
@@ -1925,6 +1977,7 @@ _make_highlight_buttons_sensitive (gpointer data)
 
   thunar_properties_dialog_update_apply_button (dialog);
 
+  g_object_unref (dialog->highlight_change_job);
   dialog->highlight_change_job = NULL;
   dialog->highlight_change_job_finish_signal = 0;
 }
