@@ -39,7 +39,6 @@
 #include <glib.h>
 #include <glib/gstdio.h>
 
-#include "thunar/thunar-file-monitor.h"
 #include "thunar/thunar-file.h"
 #include "thunar/thunar-shortcuts-model.h"
 #include "thunar/thunar-device-monitor.h"
@@ -124,11 +123,11 @@ static gboolean           thunar_shortcuts_model_get_hidden         (ThunarShort
                                                                      ThunarShortcut            *shortcut);
 static void               thunar_shortcuts_model_add_shortcut       (ThunarShortcutsModel      *model,
                                                                      ThunarShortcut            *shortcut);
-static void               thunar_shortcuts_model_remove_shortcut    (ThunarShortcutsModel      *model,
-                                                                     ThunarShortcut            *shortcut);
-static gboolean           thunar_shortcuts_model_load               (gpointer                   data);
-static void               thunar_shortcuts_model_save               (ThunarShortcutsModel      *model);
-static void               thunar_shortcuts_model_monitor            (GFileMonitor              *monitor,
+static void               thunar_shortcuts_model_remove_shortcut    (ThunarShortcut            *shortcut, 
+                                                                     ThunarShortcutsModel      *model);
+static gboolean           thunar_shortcuts_model_load_bookmarks     (gpointer                   data);
+static void               thunar_shortcuts_model_save_bookmarks     (ThunarShortcutsModel      *model);
+static void               thunar_shortcuts_model_monitor_bookmarks_file (GFileMonitor              *monitor,
                                                                      GFile                     *file,
                                                                      GFile                     *other_file,
                                                                      GFileMonitorEvent          event_type,
@@ -142,11 +141,9 @@ static void               thunar_shortcuts_model_device_removed     (ThunarDevic
 static void               thunar_shortcuts_model_device_changed     (ThunarDeviceMonitor       *device_monitor,
                                                                      ThunarDevice              *device,
                                                                      ThunarShortcutsModel      *model);
-static void               thunar_shortcuts_model_file_changed       (ThunarFileMonitor         *file_monitor,
-                                                                     ThunarFile                *file,
+static void               thunar_shortcuts_model_file_changed       (ThunarFile                *file,
                                                                      ThunarShortcutsModel      *model);
-static void               thunar_shortcuts_model_file_destroyed     (ThunarFileMonitor         *file_monitor,
-                                                                     ThunarFile                *file,
+static void               thunar_shortcuts_model_file_destroyed     (ThunarFile                *file,
                                                                      ThunarShortcutsModel      *model);
 static void               thunar_shortcut_free                      (ThunarShortcut            *shortcut,
                                                                      ThunarShortcutsModel      *model);
@@ -176,7 +173,6 @@ struct _ThunarShortcutsModel
   gchar               **hidden_bookmarks;
   gboolean              file_size_binary;
 
-  ThunarFileMonitor    *file_monitor;
   ThunarDeviceMonitor  *device_monitor;
 
   gint64                bookmarks_time;
@@ -303,11 +299,6 @@ thunar_shortcuts_model_init (ThunarShortcutsModel *model)
 
   /* add bookmarks */
   thunar_shortcuts_model_shortcut_places (model);
-
-  /* connect to the file monitor */
-  model->file_monitor = thunar_file_monitor_get_default ();
-  g_signal_connect (G_OBJECT (model->file_monitor), "file-changed", G_CALLBACK (thunar_shortcuts_model_file_changed), model);
-  g_signal_connect (G_OBJECT (model->file_monitor), "file-destroyed", G_CALLBACK (thunar_shortcuts_model_file_destroyed), model);
 }
 
 
@@ -315,6 +306,7 @@ thunar_shortcuts_model_init (ThunarShortcutsModel *model)
 static void
 thunar_shortcuts_model_finalize (GObject *object)
 {
+
   ThunarShortcutsModel *model = THUNAR_SHORTCUTS_MODEL (object);
 
   _thunar_return_if_fail (THUNAR_IS_SHORTCUTS_MODEL (model));
@@ -337,11 +329,6 @@ thunar_shortcuts_model_finalize (GObject *object)
   /* free hidden list */
   g_strfreev (model->hidden_bookmarks);
 
-  /* disconnect from the file monitor */
-  g_signal_handlers_disconnect_by_func (model->file_monitor, thunar_shortcuts_model_file_changed, model);
-  g_signal_handlers_disconnect_by_func (model->file_monitor, thunar_shortcuts_model_file_destroyed, model);
-  g_object_unref (model->file_monitor);
-
   /* detach from the file monitor */
   if (model->bookmarks_monitor != NULL)
     {
@@ -1139,11 +1126,12 @@ thunar_shortcuts_model_shortcut_places (ThunarShortcutsModel *model)
   if (G_LIKELY (model->bookmarks_monitor != NULL))
     {
       g_signal_connect (model->bookmarks_monitor, "changed",
-        G_CALLBACK (thunar_shortcuts_model_monitor), model);
+        G_CALLBACK (thunar_shortcuts_model_monitor_bookmarks_file), model);
     }
 
   /* read the Gtk+ bookmarks file */
-  model->bookmarks_idle_id = g_idle_add_full (G_PRIORITY_DEFAULT, thunar_shortcuts_model_load, model, NULL);
+  if (model->bookmarks_idle_id == 0)
+    model->bookmarks_idle_id = g_idle_add_full (G_PRIORITY_DEFAULT, thunar_shortcuts_model_load_bookmarks, model, NULL);
 
   /* append the computer icon if browsing the computer is supported */
   if (thunar_g_vfs_is_uri_scheme_supported ("computer"))
@@ -1248,9 +1236,12 @@ thunar_shortcuts_model_add_shortcut_with_path (ThunarShortcutsModel *model,
 
   if (G_LIKELY (shortcut->file != NULL))
     {
-      /* watch the trash for changes */
-      if (thunar_file_is_trash (shortcut->file))
-        thunar_file_watch (shortcut->file);
+      /* enable monitoring for the file */
+      thunar_file_watch (shortcut->file);
+
+      /* subscribe to relevant signals */
+      g_signal_connect (G_OBJECT (shortcut->file), "destroy", G_CALLBACK (thunar_shortcuts_model_file_destroyed), model);
+      g_signal_connect (G_OBJECT (shortcut->file), "changed", G_CALLBACK (thunar_shortcuts_model_file_changed), model);
     }
 
   if (path == NULL)
@@ -1288,12 +1279,11 @@ thunar_shortcuts_model_add_shortcut (ThunarShortcutsModel *model,
 
 
 static void
-thunar_shortcuts_model_remove_shortcut (ThunarShortcutsModel *model,
-                                        ThunarShortcut       *shortcut)
+thunar_shortcuts_model_remove_shortcut (ThunarShortcut       *shortcut,
+                                        ThunarShortcutsModel *model)
 {
   GtkTreePath *path;
   gint         idx;
-  gboolean     needs_save;
 
   /* determine the index of the shortcut */
   idx = g_list_index (model->shortcuts, shortcut);
@@ -1307,16 +1297,9 @@ thunar_shortcuts_model_remove_shortcut (ThunarShortcutsModel *model,
       gtk_tree_model_row_deleted (GTK_TREE_MODEL (model), path);
       gtk_tree_path_free (path);
 
-      /* check if we need to save */
-      needs_save = (shortcut->group == THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS);
-
       /* actually free the shortcut */
       thunar_shortcut_free (shortcut, model);
 
-      /* the shortcuts list was changed, so write the gtk bookmarks file */
-      if (needs_save)
-        thunar_shortcuts_model_save (model);
-
       /* update header visibility */
       thunar_shortcuts_model_header_visibility (model);
     }
@@ -1385,7 +1368,7 @@ thunar_shortcuts_model_load_line (GFile       *file_path,
 
 
 static gboolean
-thunar_shortcuts_model_load (gpointer data)
+thunar_shortcuts_model_load_bookmarks (gpointer data)
 {
   ThunarShortcutsModel *model = THUNAR_SHORTCUTS_MODEL (data);
 
@@ -1407,57 +1390,43 @@ thunar_shortcuts_model_load (gpointer data)
 
 
 static gboolean
-thunar_shortcuts_model_reload (gpointer data)
+thunar_shortcuts_model_reload_bookmarks (gpointer data)
 {
   ThunarShortcutsModel *model = THUNAR_SHORTCUTS_MODEL (data);
-  ThunarShortcut       *shortcut;
-  GtkTreePath          *path;
   GList                *lp;
-  gint                  idx;
+  GList                *places_bookmarks = NULL;
 
   _thunar_return_val_if_fail (THUNAR_IS_SHORTCUTS_MODEL (model), FALSE);
 
-  /* drop all existing user-defined shortcuts from the model */
-  for (idx = 0, lp = model->shortcuts; lp != NULL; )
+  /* Find all user-defined shortcuts (bookmarks) */
+  for (lp = model->shortcuts; lp != NULL; lp = lp->next)
     {
-      /* grab the shortcut */
-      shortcut = THUNAR_SHORTCUT (lp->data);
-
-      /* advance to the next list item */
-      lp = g_list_next (lp);
-
-      /* drop the shortcut if it is user-defined */
+      ThunarShortcut *shortcut = THUNAR_SHORTCUT (lp->data);
       if (shortcut->group == THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS)
-        {
-          /* unlink the shortcut from the model */
-          model->shortcuts = g_list_remove (model->shortcuts, shortcut);
-
-          /* tell everybody that we have lost a shortcut */
-          path = gtk_tree_path_new_from_indices (idx, -1);
-          gtk_tree_model_row_deleted (GTK_TREE_MODEL (model), path);
-          gtk_tree_path_free (path);
-
-          /* actually free the shortcut */
-          thunar_shortcut_free (shortcut, model);
-        }
-      else
-        {
-          ++idx;
-        }
+        places_bookmarks = g_list_append (places_bookmarks, shortcut);
     }
 
-  /* load new bookmarks */
-  return thunar_shortcuts_model_load (data);
+  /* drop them from the model */
+  g_list_foreach (places_bookmarks, (GFunc) (void (*)(void)) thunar_shortcuts_model_remove_shortcut, model);
+  g_list_free (places_bookmarks);
+
+  /* reload them from the bookmarks file */
+  thunar_shortcuts_model_load_bookmarks (model);
+
+  /* reset the idle ID */
+  model->bookmarks_idle_id = 0;
+
+  return FALSE;
 }
 
 
 
 static void
-thunar_shortcuts_model_monitor (GFileMonitor     *monitor,
-                                GFile            *file,
-                                GFile            *other_file,
-                                GFileMonitorEvent event_type,
-                                gpointer          user_data)
+thunar_shortcuts_model_monitor_bookmarks_file (GFileMonitor     *monitor,
+                                               GFile            *file,
+                                               GFile            *other_file,
+                                               GFileMonitorEvent event_type,
+                                               gpointer          user_data)
 {
   ThunarShortcutsModel *model = THUNAR_SHORTCUTS_MODEL (user_data);
 
@@ -1468,15 +1437,15 @@ thunar_shortcuts_model_monitor (GFileMonitor     *monitor,
   if (model->bookmarks_time + 2 * G_USEC_PER_SEC > g_get_real_time ())
     return;
 
-  /* reload the shortcuts model */
+  /* reload the bookmarks when idle */
   if (model->bookmarks_idle_id == 0)
-    model->bookmarks_idle_id = g_idle_add (thunar_shortcuts_model_reload, model);
+    model->bookmarks_idle_id = g_idle_add (thunar_shortcuts_model_reload_bookmarks, model);
 }
 
 
 
 static void
-thunar_shortcuts_model_save (ThunarShortcutsModel *model)
+thunar_shortcuts_model_save_bookmarks (ThunarShortcutsModel *model)
 {
   GString        *contents;
   ThunarShortcut *shortcut;
@@ -1616,7 +1585,7 @@ thunar_shortcuts_model_device_removed (ThunarDeviceMonitor  *device_monitor,
 
   /* drop the shortcut from the model */
   if (G_LIKELY (lp != NULL))
-    thunar_shortcuts_model_remove_shortcut (model, lp->data);
+    thunar_shortcuts_model_remove_shortcut (lp->data, model);
 }
 
 
@@ -1662,6 +1631,11 @@ thunar_shortcuts_model_device_changed (ThunarDeviceMonitor  *device_monitor,
           if (mount_point != NULL)
             {
               shortcut->file = thunar_file_get (mount_point, NULL);
+
+              /* enable monitoring for the file */
+              if (shortcut->file != NULL)
+                thunar_file_watch (shortcut->file);
+
               g_object_unref (mount_point);
             }
         }
@@ -1690,8 +1664,7 @@ thunar_shortcuts_model_device_changed (ThunarDeviceMonitor  *device_monitor,
 
 
 static void
-thunar_shortcuts_model_file_destroyed (ThunarFileMonitor      *file_monitor,
-                                       ThunarFile             *file,
+thunar_shortcuts_model_file_destroyed (ThunarFile             *file,
                                        ThunarShortcutsModel   *model)
 {
   GtkTreeIter     iter;
@@ -1701,35 +1674,48 @@ thunar_shortcuts_model_file_destroyed (ThunarFileMonitor      *file_monitor,
 
     /* find the shortcut */
   for (lp = model->shortcuts, idx = 0; lp != NULL; lp = lp->next, idx++)
-    if (THUNAR_SHORTCUT (lp->data)->file == file)
-      break;
-
-  if (G_LIKELY (lp != NULL))
     {
-      /* remove the thunar file from the list and set the g_file instead */
-      GFile *g_file = thunar_file_get_file (file);
-      THUNAR_SHORTCUT (lp->data)->file = NULL;
-      THUNAR_SHORTCUT (lp->data)->location = g_file;
-      if (thunar_shortcuts_model_local_file (g_file))
-        THUNAR_SHORTCUT (lp->data)->gicon = g_themed_icon_new ("folder");
-      else
-        THUNAR_SHORTCUT (lp->data)->gicon = g_themed_icon_new ("folder-remote");
+      ThunarShortcut *shortcut = THUNAR_SHORTCUT (lp->data);
+      if (shortcut->file == file)
+        {
+          /* Disconnect from relevant signals */
+          thunar_file_unwatch (file);
+          g_signal_handlers_disconnect_by_data (shortcut->file, model);
+          g_object_unref (shortcut->file);
+          shortcut->file = NULL;
 
-      /* generate an iterator for the path */
-      GTK_TREE_ITER_INIT (iter, model->stamp, lp);
+          /* remove the thunar file from the list and set the g_file instead */
+          if (shortcut->location == NULL)
+            { 
+              GFile *g_file = thunar_file_get_file (file);
+              shortcut->location = g_file;
+            }
 
-      /* tell the view that the file has changed in some way */
-      path = gtk_tree_path_new_from_indices (idx, -1);
-      gtk_tree_model_row_changed (GTK_TREE_MODEL (model), path, &iter);
-      gtk_tree_path_free (path);
+          if (shortcut->gicon == NULL)
+          {
+            if (thunar_shortcuts_model_local_file (shortcut->location))
+              shortcut->gicon = g_themed_icon_new ("folder");
+            else
+              shortcut->gicon = g_themed_icon_new ("folder-remote");
+          }
+
+          /* generate an iterator for the path */
+          GTK_TREE_ITER_INIT (iter, model->stamp, lp);
+
+          /* tell the view that the file has changed in some way */
+          path = gtk_tree_path_new_from_indices (idx, -1);
+          gtk_tree_model_row_changed (GTK_TREE_MODEL (model), path, &iter);
+          gtk_tree_path_free (path);
+
+          break;
+        }
     }
 }
 
 
 
 static void
-thunar_shortcuts_model_file_changed (ThunarFileMonitor      *file_monitor,
-                                     ThunarFile             *file,
+thunar_shortcuts_model_file_changed (ThunarFile             *file,
                                      ThunarShortcutsModel   *model)
 {
   GtkTreeIter     iter;
@@ -1737,24 +1723,22 @@ thunar_shortcuts_model_file_changed (ThunarFileMonitor      *file_monitor,
   gint            idx;
   GtkTreePath    *path;
 
-  _thunar_return_if_fail (THUNAR_IS_FILE_MONITOR (file_monitor));
-  _thunar_return_if_fail (model->file_monitor == file_monitor);
   _thunar_return_if_fail (THUNAR_IS_SHORTCUTS_MODEL (model));
   _thunar_return_if_fail (THUNAR_IS_FILE (file));
 
-    /* find the shortcut */
-  for (lp = model->shortcuts, idx = 0; lp != NULL; lp = lp->next, idx++)
-    if (THUNAR_SHORTCUT (lp->data)->file == file)
-      break;
 
-  if (G_LIKELY (lp != NULL))
+  for (lp = model->shortcuts, idx = 0; lp != NULL; lp = lp->next, idx++)
     {
+      ThunarShortcut *shortcut = THUNAR_SHORTCUT (lp->data);
+
+      if (shortcut->file != file)
+        continue;
 
       GFile *g_file = thunar_file_get_file (file);
 
       /* Only update the shortcut if trash has changed or the related g_file has changed (rename) */
       if (thunar_file_is_trash (file) ||
-          (THUNAR_SHORTCUT (lp->data)->location != NULL && g_file_equal (g_file, THUNAR_SHORTCUT (lp->data)->location) == FALSE))
+          (shortcut->location != NULL && g_file_equal (g_file, shortcut->location) == FALSE))
         {
           /* generate an iterator for the path */
           GTK_TREE_ITER_INIT (iter, model->stamp, lp);
@@ -1764,9 +1748,11 @@ thunar_shortcuts_model_file_changed (ThunarFileMonitor      *file_monitor,
           gtk_tree_model_row_changed (GTK_TREE_MODEL (model), path, &iter);
           gtk_tree_path_free (path);
 
-          /* Store the changed shortcut */
-          thunar_shortcuts_model_save (model);
+          /* If the changed shortcut is a bookmark, save the changes */
+          if (shortcut->group == THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS)
+            thunar_shortcuts_model_save_bookmarks (model);
         }
+      break;
     }
 }
 
@@ -1778,11 +1764,11 @@ thunar_shortcut_free (ThunarShortcut       *shortcut,
 {
   if (G_LIKELY (shortcut->file != NULL))
     {
-      /* drop the file watch on trash */
-      if (thunar_file_is_trash (shortcut->file))
-        thunar_file_unwatch (shortcut->file);
-
+      /* disable monitoring for the file */
+      thunar_file_unwatch (shortcut->file);
+      g_signal_handlers_disconnect_by_data (shortcut->file, model);
       g_object_unref (shortcut->file);
+      shortcut->file = NULL;
     }
 
   if (G_LIKELY (shortcut->device != NULL))
@@ -2089,7 +2075,7 @@ thunar_shortcuts_model_add (ThunarShortcutsModel *model,
   thunar_shortcuts_model_add_shortcut_with_path (model, shortcut, dst_path);
 
   /* the shortcuts list was changed, so write the gtk bookmarks file */
-  thunar_shortcuts_model_save (model);
+  thunar_shortcuts_model_save_bookmarks (model);
 }
 
 
@@ -2183,7 +2169,7 @@ thunar_shortcuts_model_move (ThunarShortcutsModel *model,
   gtk_tree_path_free (path);
 
   /* the shortcuts list was changed, so write the gtk bookmarks file */
-  thunar_shortcuts_model_save (model);
+  thunar_shortcuts_model_save_bookmarks (model);
 }
 
 
@@ -2216,7 +2202,10 @@ thunar_shortcuts_model_remove (ThunarShortcutsModel *model,
   _thunar_assert (shortcut->group == THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS);
 
   /* remove the shortcut (using the file destroy handler) */
-  thunar_shortcuts_model_remove_shortcut (model, shortcut);
+  thunar_shortcuts_model_remove_shortcut (shortcut, model);
+
+  /* the bookmark list was changed, so write the gtk bookmarks file */
+  thunar_shortcuts_model_save_bookmarks (model);
 }
 
 
@@ -2267,7 +2256,7 @@ thunar_shortcuts_model_rename (ThunarShortcutsModel *model,
   gtk_tree_path_free (path);
 
   /* save the changes to the model */
-  thunar_shortcuts_model_save (model);
+  thunar_shortcuts_model_save_bookmarks (model);
 }
 
 
