@@ -203,6 +203,9 @@ struct _ThunarShortcutsView
   /* used to create menu items for the context menu */
   ThunarActionManager *action_mgr;
   ThunarFile          *current_directory;
+
+  /* reference to the underlying model */
+  ThunarShortcutsModel *model;
 };
 
 
@@ -355,6 +358,10 @@ thunar_shortcuts_view_init (ThunarShortcutsView *view)
   g_object_bind_property (G_OBJECT (view->preferences), "shortcuts-icon-size", G_OBJECT (view->icon_renderer), "size", G_BINDING_SYNC_CREATE);
   g_object_bind_property (G_OBJECT (view->preferences), "shortcuts-icon-emblems", G_OBJECT (view->icon_renderer), "emblems", G_BINDING_SYNC_CREATE);
 
+  /* optional symbolic icons */
+  g_object_bind_property (G_OBJECT (view->preferences), "misc-symbolic-icons-in-sidepane", G_OBJECT (view->icon_renderer), "use-symbolic-icons", G_BINDING_SYNC_CREATE);
+  g_signal_connect_swapped (G_OBJECT (view->preferences), "notify::misc-symbolic-icons-in-sidepane", G_CALLBACK (gtk_widget_queue_draw), view);
+
   /* allocate the text renderer (ellipsizing as required, but "File System" must fit) */
   renderer = g_object_new (GTK_TYPE_CELL_RENDERER_TEXT,
                            "ellipsize", PANGO_ELLIPSIZE_END,
@@ -420,7 +427,13 @@ thunar_shortcuts_view_init (ThunarShortcutsView *view)
 static void
 thunar_shortcuts_view_finalize (GObject *object)
 {
-  ThunarShortcutsView *view = THUNAR_SHORTCUTS_VIEW (object);
+  ThunarShortcutsView  *view = THUNAR_SHORTCUTS_VIEW (object);
+
+  /* Disconnect from the model */
+  g_signal_handlers_disconnect_by_data (G_OBJECT (view->model), view);
+
+  /* release our reference on the model */
+  g_object_unref (view->model);
 
   /* release drop path list (if drag_leave wasn't called) */
   thunar_g_list_free_full (view->drop_file_list);
@@ -1283,9 +1296,9 @@ thunar_shortcuts_view_context_menu (ThunarShortcutsView *view,
         }
       else
         {
-          thunar_menu_add_sections (context_menu, THUNAR_MENU_SECTION_OPEN);
-          thunar_menu_add_sections (context_menu, THUNAR_MENU_SECTION_SENDTO
-                                                | THUNAR_MENU_SECTION_CREATE_NEW_FILES
+          thunar_menu_add_sections (context_menu, THUNAR_MENU_SECTION_OPEN
+                                                | THUNAR_MENU_SECTION_SENDTO);
+          thunar_menu_add_sections (context_menu, THUNAR_MENU_SECTION_CREATE_NEW_FILES
                                                 | THUNAR_MENU_SECTION_COPY_PASTE
                                                 | THUNAR_MENU_SECTION_CUSTOM_ACTIONS);
         }
@@ -1829,9 +1842,13 @@ thunar_shortcuts_view_new (void)
   model = thunar_shortcuts_model_get_default ();
   filter_model = gtk_tree_model_filter_new (GTK_TREE_MODEL (model), NULL);
   gtk_tree_model_filter_set_visible_column (GTK_TREE_MODEL_FILTER (filter_model), THUNAR_SHORTCUTS_MODEL_COLUMN_VISIBLE);
-  g_object_unref (G_OBJECT (model));
 
   view = g_object_new (THUNAR_TYPE_SHORTCUTS_VIEW, "model", filter_model, NULL);
+  THUNAR_SHORTCUTS_VIEW (view)->model = model;
+
+  /* redraw the view when the model changes */
+  g_signal_connect_swapped (G_OBJECT (model), "row-changed", G_CALLBACK (gtk_widget_queue_draw), view);
+
   g_object_unref (G_OBJECT (filter_model));
 
   return view;
