@@ -20,19 +20,19 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <exo/exo.h>
 #include <libxfce4ui/libxfce4ui.h>
 
-#include <thunar/thunar-gobject-extensions.h>
-#include <thunar/thunar-private.h>
-#include <thunar/thunar-statusbar.h>
-#include <thunar/thunar-gtk-extensions.h>
-#include <thunar/thunar-preferences.h>
-#include <thunar/thunar-application.h>
-#include <thunar/thunar-window.h>
+#include "thunar/thunar-gobject-extensions.h"
+#include "thunar/thunar-private.h"
+#include "thunar/thunar-statusbar.h"
+#include "thunar/thunar-gtk-extensions.h"
+#include "thunar/thunar-preferences.h"
+#include "thunar/thunar-application.h"
+#include "thunar/thunar-window.h"
 
 
 
