@@ -94,6 +94,18 @@ typedef enum
 
 
 
+enum
+{
+    DBUS_THUMBNAIL_ERROR_UNSUPPORTED,
+    DBUS_THUMBNAIL_ERROR_CONNECTION_ERROR,
+    DBUS_THUMBNAIL_ERROR_INVALID_FORMAT,
+    DBUS_THUMBNAIL_ERROR_IS_THUMBNAIL,
+    DBUS_THUMBNAIL_ERROR_SAVE_FAILED,
+    DBUS_THUMBNAIL_ERROR_UNSUPPORTED_FLAVOR,
+};
+
+
+
 typedef struct _ThunarThumbnailerJob  ThunarThumbnailerJob;
 
 /* Signal identifiers */
@@ -364,7 +376,7 @@ thunar_thumbnailer_queue_async_reply (GObject      *proxy,
     {
       if (handle == 0)
         {
-          g_printerr ("ThunarThumbnailer: got 0 handle (Queue)\n");
+          g_warning ("ThunarThumbnailer: got 0 handle (Queue)");
         }
       else
         {
@@ -374,7 +386,7 @@ thunar_thumbnailer_queue_async_reply (GObject      *proxy,
     }
   else
     {
-      g_printerr ("ThunarThumbnailer: Queue failed: %s\n", error->message);
+      g_warning ("ThunarThumbnailer: Queue failed: %s", error->message);
     }
 
   _thumbnailer_unlock (thumbnailer);
@@ -654,7 +666,7 @@ thunar_thumbnailer_received_supported_types (ThunarThumbnailerDBus  *proxy,
       g_slist_free_full (thumbnailer->jobs, (GDestroyNotify)thunar_thumbnailer_free_job);
       thumbnailer->jobs = NULL;
 
-      g_printerr ("ThunarThumbnailer: Failed to retrieve supported types: %s\n", error->message);
+      g_warning ("ThunarThumbnailer: Failed to retrieve supported types: %s", error->message);
       g_clear_error (&error);
 
       thumbnailer->proxy_state = THUNAR_THUMBNAILER_PROXY_FAILED;
@@ -736,7 +748,7 @@ thunar_thumbnailer_proxy_created (GObject       *object,
       _thumbnailer_lock (thumbnailer);
 
       thumbnailer->proxy_state = THUNAR_THUMBNAILER_PROXY_FAILED;
-      g_printerr ("ThunarThumbnailer: failed to create proxy: %s", error->message);
+      g_warning ("ThunarThumbnailer: failed to create proxy: %s", error->message);
       g_clear_error (&error);
 
       g_slist_free_full (thumbnailer->jobs, (GDestroyNotify)thunar_thumbnailer_free_job);
@@ -878,7 +890,7 @@ thunar_thumbnailer_thumbnailer_error (GDBusProxy        *proxy,
   for (GSList *lp = thumbnailer->jobs; lp != NULL; lp = lp->next)
     {
       job = lp->data;
-      if (job->handle != handle)
+      if (job->handle == handle)
         {
           for (const gchar **uri = uris; *uri != NULL; ++uri)
             {
@@ -890,9 +902,19 @@ thunar_thumbnailer_thumbnailer_error (GDBusProxy        *proxy,
               /* check if we have a file for this URI in the cache */
               if (file != NULL)
                 {
-                  /* tell everybody we're done here */
-                  thunar_file_update_thumbnail (file, THUNAR_FILE_THUMB_STATE_NONE, job->thumbnail_size);
-                  g_debug ("Failed to generate thumbnail for '%s': Error Code: %i - Error: %s\n", thunar_file_get_basename (file), code, message);
+                  if (code == DBUS_THUMBNAIL_ERROR_IS_THUMBNAIL)
+                    {
+                      /* the file itself is a thumbnail, so use it as a thumbnail */
+                      thunar_file_set_is_thumbnail (file, TRUE);
+                      thunar_file_update_thumbnail (file, THUNAR_FILE_THUMB_STATE_READY, job->thumbnail_size);
+                    }
+                  else
+                    {
+                      /* tell everybody we're done here */
+                      thunar_file_update_thumbnail (file, THUNAR_FILE_THUMB_STATE_NONE, job->thumbnail_size);
+                      g_debug ("Failed to generate thumbnail for '%s': Error Code: %i - Error: %s\n", thunar_file_get_basename (file), code, message);
+                    }
+
                   g_object_unref (file);
                 }
             }
@@ -957,7 +979,7 @@ thunar_thumbnailer_thumbnailer_finished (GDBusProxy        *proxy,
 
   if (handle == 0)
     {
-      g_printerr ("ThunarThumbnailer: got 0 handle (Finished)\n");
+      g_warning ("ThunarThumbnailer: got 0 handle (Finished)");
       return;
     }
 
