@@ -55,6 +55,7 @@ static void     thunar_toolbar_editor_toggle_visibility                  (Thunar
 static void     thunar_toolbar_editor_update_buttons                     (ThunarToolbarEditor      *toolbar_editor);
 static void     thunar_toolbar_editor_use_defaults                       (ThunarToolbarEditor      *toolbar_editor,
                                                                           GtkWidget                *button);
+static void     thunar_toolbar_editor_toggle_symbolic_icons              (ThunarToolbarEditor      *toolbar_editor);
 static void     thunar_toolbar_editor_save_model                         (ThunarToolbarEditor      *toolbar_editor);
 static void     thunar_toolbar_editor_populate_model                     (ThunarToolbarEditor      *toolbar_editor);
 
@@ -109,18 +110,23 @@ thunar_toolbar_editor_init (ThunarToolbarEditor *toolbar_editor)
   GtkWidget              *vbox;
   GtkWidget              *hbox;
   GtkWidget              *vbox_buttons;
+  GtkWidget              *separator;
   GtkWidget              *swin;
 
   /* grab a reference on the preferences */
   toolbar_editor->preferences = thunar_preferences_get ();
 
   /* grab a reference on the shared toolbar model */
-  toolbar_editor->model = gtk_list_store_new (4, G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT);
+  toolbar_editor->model = gtk_list_store_new (4,
+                                              G_TYPE_STRING,  /* action id */
+                                              G_TYPE_BOOLEAN, /* visibility */
+                                              G_TYPE_STRING,  /* icon name */
+                                              G_TYPE_STRING); /* label */
 
   /* setup the dialog */
   gtk_dialog_add_button (GTK_DIALOG (toolbar_editor), _("_Close"), GTK_RESPONSE_CLOSE);
   gtk_dialog_set_default_response (GTK_DIALOG (toolbar_editor), GTK_RESPONSE_CLOSE);
-  gtk_window_set_default_size (GTK_WINDOW (toolbar_editor), 510, 490);
+  gtk_window_set_default_size (GTK_WINDOW (toolbar_editor), 450, 500);
   gtk_window_set_resizable (GTK_WINDOW (toolbar_editor), TRUE);
   gtk_window_set_title (GTK_WINDOW (toolbar_editor), _("Configure the Toolbar"));
 
@@ -137,7 +143,9 @@ thunar_toolbar_editor_init (ThunarToolbarEditor *toolbar_editor)
   gtk_widget_show (vbox);
 
   /* create the top label for the toolbar editor dialog */
-  label = gtk_label_new (_("Configure the order and visibility of toolbar items."));
+  label = gtk_label_new (_("Configure the order and visibility of toolbar items.\n"
+                           "Note that toolbar items are always executed for the current directory."));
+  gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
   gtk_label_set_xalign (GTK_LABEL (label), 0.0f);
   gtk_widget_set_hexpand (label, TRUE);
   gtk_widget_set_vexpand (label, FALSE);
@@ -166,36 +174,36 @@ thunar_toolbar_editor_init (ThunarToolbarEditor *toolbar_editor)
   /* create the tree view */
   toolbar_editor->tree_view = gtk_tree_view_new_with_model (toolbar_editor->filter);
   gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (toolbar_editor->tree_view), FALSE);
-  gtk_tree_view_set_search_column (GTK_TREE_VIEW (toolbar_editor->tree_view), 2);
+  gtk_tree_view_set_search_column (GTK_TREE_VIEW (toolbar_editor->tree_view), 3);
   gtk_container_add (GTK_CONTAINER (swin), toolbar_editor->tree_view);
   gtk_widget_show (toolbar_editor->tree_view);
 
-  /* append the toggle toolbar */
+  /* append the toggle visibility column */
   renderer = gtk_cell_renderer_toggle_new ();
   g_signal_connect_swapped (G_OBJECT (renderer), "toggled", G_CALLBACK (thunar_toolbar_editor_toggle_visibility), toolbar_editor);
   gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (toolbar_editor->tree_view),
                                                -1,
                                                "Active",
                                                renderer,
-                                               "active", 0,
+                                               "active", 1,
                                                NULL);
 
-  /* append the icon toolbar */
+  /* append the icon column */
   renderer = gtk_cell_renderer_pixbuf_new ();
   gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (toolbar_editor->tree_view),
                                                -1,
                                                "Icon",
                                                renderer,
-                                               "icon-name", 1,
+                                               "icon-name", 2,
                                                NULL);
 
-  /* append the name toolbar */
+  /* append the name column */
   renderer = gtk_cell_renderer_text_new ();
   gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (toolbar_editor->tree_view),
                                                -1,
                                                "Name",
                                                renderer,
-                                               "text", 2,
+                                               "text", 3,
                                                NULL);
 
   /* Create the buttons vbox container */
@@ -226,12 +234,31 @@ thunar_toolbar_editor_init (ThunarToolbarEditor *toolbar_editor)
   gtk_button_set_image (GTK_BUTTON (toolbar_editor->down_button), image);
   gtk_widget_show (image);
 
+  /* add separator */
+  separator = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
+  gtk_box_pack_start (GTK_BOX (vbox_buttons), separator, FALSE, FALSE, 0);
+  gtk_widget_show (separator);
+
   /* create the "Use Default" button */
   button = gtk_button_new_with_mnemonic (_("De_fault Order"));
   g_signal_connect_swapped (G_OBJECT (button), "clicked", G_CALLBACK (thunar_toolbar_editor_use_defaults), toolbar_editor);
   gtk_box_pack_start (GTK_BOX (vbox_buttons), button, FALSE, FALSE, 0);
   gtk_widget_show (button);
 
+  /* create the "Symbolic Icons" checkbox */
+  button = gtk_check_button_new_with_mnemonic (_("_Symbolic Icons"));
+  g_object_bind_property (G_OBJECT (toolbar_editor->preferences),
+                          "misc-symbolic-icons-in-toolbar",
+                          G_OBJECT (button),
+                          "active",
+                          G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
+  gtk_widget_set_tooltip_text (button, _("Use symbolic icons instead of regular ones (if available)."));
+  gtk_box_pack_end (GTK_BOX (vbox_buttons), button, FALSE, FALSE, 0);
+  gtk_widget_show (button);
+
+  g_signal_connect_swapped (G_OBJECT (toolbar_editor->preferences), "notify::misc-symbolic-icons-in-toolbar",
+                            G_CALLBACK (thunar_toolbar_editor_toggle_symbolic_icons), toolbar_editor);
+
   /* grab focus to the tree view */
   gtk_widget_grab_focus (toolbar_editor->tree_view);
 }
@@ -250,6 +277,7 @@ thunar_toolbar_editor_finalize (GObject *object)
   g_object_unref (G_OBJECT (toolbar_editor->model));
 
   /* release our reference on the preferences */
+  g_signal_handlers_disconnect_by_data (G_OBJECT (toolbar_editor->preferences), toolbar_editor);
   g_object_unref (G_OBJECT (toolbar_editor->preferences));
 
   (*G_OBJECT_CLASS (thunar_toolbar_editor_parent_class)->finalize) (object);
@@ -260,17 +288,19 @@ thunar_toolbar_editor_finalize (GObject *object)
 static gboolean
 thunar_toolbar_editor_visible_func (GtkTreeModel *model,
                                     GtkTreeIter  *iter,
-                                    gpointer     data)
+                                    gpointer      data)
 {
-  gboolean visible = TRUE;
-  gint     order;
+  gboolean  visible = TRUE;
+  gchar    *id;
 
-  gtk_tree_model_get (model, iter, 3, &order, -1);
+  gtk_tree_model_get (model, iter, 0, &id, -1);
 
-   /* The item with order 0 is always THUNAR_WINDOW_ACTION_VIEW_MENUBAR which we want to hide */
-  if (order == 0)
+  /* Always hide THUNAR_WINDOW_ACTION_VIEW_MENUBAR */
+  if (g_strcmp0 (id, "view-menubar") == 0)
     visible = FALSE;
 
+  g_free (id);
+
   return visible;
 }
 
@@ -416,8 +446,8 @@ thunar_toolbar_editor_toggle_visibility (ThunarToolbarEditor    *toolbar_editor,
 
   if (gtk_tree_model_get_iter (GTK_TREE_MODEL (toolbar_editor->model), &iter, child_path))
     {
-      gtk_tree_model_get (GTK_TREE_MODEL (toolbar_editor->model), &iter, 0, &visible, -1);
-      gtk_list_store_set (toolbar_editor->model, &iter, 0, !visible, -1);
+      gtk_tree_model_get (GTK_TREE_MODEL (toolbar_editor->model), &iter, 1, &visible, -1);
+      gtk_list_store_set (toolbar_editor->model, &iter, 1, !visible, -1);
     }
 
   application = thunar_application_get ();
@@ -533,60 +563,68 @@ thunar_toolbar_editor_use_defaults (ThunarToolbarEditor *toolbar_editor,
 
 
 
+static void
+thunar_toolbar_editor_toggle_symbolic_icons (ThunarToolbarEditor *toolbar_editor)
+{
+  /* clear the tree view */
+  gtk_list_store_clear (toolbar_editor->model);
+  g_signal_handlers_disconnect_by_data (G_OBJECT (toolbar_editor->model), toolbar_editor);
+
+  /* populate the tree view */
+  thunar_toolbar_editor_populate_model (toolbar_editor);
+}
+
+
+
 static void
 thunar_toolbar_editor_save_model (ThunarToolbarEditor *toolbar_editor)
 {
-  GString    *item_order;
-  GString    *item_visibility;
-  guint       item_count;
+  GString *items;
+  guint    item_count;
 
-  item_order = g_string_sized_new (256);
-  item_visibility = g_string_sized_new (256);
+  items = g_string_sized_new (512);
 
   item_count = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (toolbar_editor->model), NULL);
 
-  /* transform the internal visible column list */
+  /* read the internal id and visibility column values and store them */
   for (guint i = 0; i < item_count; i++)
     {
-      GtkTreeIter iter;
-      gchar      *path;
-      gint        order;
-      gchar      *order_str;
-      gboolean    visible;
-      gchar      *visible_str;
-
-      /* append a comma if not empty */
-      if (*item_order->str != '\0')
-        g_string_append_c (item_order, ',');
-
-      if (*item_visibility->str != '\0')
-        g_string_append_c (item_visibility, ',');
+      GtkTreeIter  iter;
+      gchar       *path;
+      gchar       *id;
+      gboolean     visible;
 
       /* get iterator for the ith item */
-      path = g_strdup_printf("%i", i);
+      path = g_strdup_printf ("%i", i);
       gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (toolbar_editor->model), &iter, path);
       g_free (path);
 
-      /* get the order value of the entry and store it */
-      gtk_tree_model_get (GTK_TREE_MODEL (toolbar_editor->model), &iter, 3, &order, -1);
-      order_str = g_strdup_printf("%i", order);
-      g_string_append (item_order, order_str);
-      g_free (order_str);
+      /* get the id value of the entry */
+      gtk_tree_model_get (GTK_TREE_MODEL (toolbar_editor->model), &iter, 0, &id, -1);
+      if (id == NULL)
+        continue;
+
+      /* append a comma if not empty */
+      if (*items->str != '\0')
+        g_string_append_c (items, ',');
+
+      /* store the id value */
+      g_string_append (items, id);
+      g_free (id);
+
+      /* append the separator character */
+      g_string_append_c (items, ':');
 
       /* get the visibility value of the entry and store it */
-      gtk_tree_model_get (GTK_TREE_MODEL (toolbar_editor->model), &iter, 0, &visible, -1);
-      visible_str = g_strdup_printf("%i", visible);
-      g_string_append (item_visibility, visible_str);
-      g_free (visible_str);
+      gtk_tree_model_get (GTK_TREE_MODEL (toolbar_editor->model), &iter, 1, &visible, -1);
+      g_string_append_printf (items, "%i", visible);
     }
 
-  /* save the order and visibility lists */
-  g_object_set (G_OBJECT (toolbar_editor->preferences), "last-toolbar-item-order", item_order->str, NULL);
-  g_object_set (G_OBJECT (toolbar_editor->preferences), "last-toolbar-visible-buttons", item_visibility->str, NULL);
+  /* save the toolbar configuration */
+  g_object_set (G_OBJECT (toolbar_editor->preferences), "last-toolbar-items", items->str, NULL);
 
-  /* release the strings */
-  g_string_free (item_order, TRUE);
-  g_string_free (item_visibility, TRUE);
+  /* release the string */
+  g_string_free (items, TRUE);
 }
 
 
@@ -604,23 +642,23 @@ thunar_toolbar_editor_populate_model (ThunarToolbarEditor *toolbar_editor)
     {
       GtkWidget *item = lp->data;
       GtkWidget *widget = NULL; /* used to remove mnemonics */
-      gchar     *label_with_mnemonic = NULL;
+      gchar     *id = NULL;
       gchar     *icon = NULL;
-      gint      *order = NULL;
+      gchar     *label_with_mnemonic = NULL;
 
-      label_with_mnemonic = g_object_get_data (G_OBJECT (item), "label");
+      id = g_object_get_data (G_OBJECT (item), "id");
       icon = g_object_get_data (G_OBJECT (item), "icon");
-      order = g_object_get_data (G_OBJECT (item), "default-order");
+      label_with_mnemonic = g_object_get_data (G_OBJECT (item), "label");
 
       widget = gtk_label_new_with_mnemonic (label_with_mnemonic);
       g_object_ref_sink (widget);
 
       gtk_list_store_append (toolbar_editor->model, &iter);
       gtk_list_store_set (toolbar_editor->model, &iter,
-                          0, gtk_widget_is_visible (item),
-                          1, icon,
-                          2, gtk_label_get_text (GTK_LABEL (widget)),
-                          3, *order,
+                          0, id,
+                          1, gtk_widget_is_visible (item),
+                          2, icon,
+                          3, gtk_label_get_text (GTK_LABEL (widget)),
                           -1);
 
       g_object_unref (widget);
@@ -696,3 +734,28 @@ thunar_show_toolbar_editor (GtkWidget *window,
   gtk_widget_destroy (dialog);
 }
 
+
+
+/**
+ * thunar_save_toolbar_configuration:
+ * @toolbar: a #GtkWidget instance.
+ *
+ * Trigger a configuration save without displaying the #ThunarToolbarEditor.
+ **/
+void
+thunar_save_toolbar_configuration (GtkWidget *toolbar)
+{
+  GtkWidget *dialog;
+
+  /* create the toolbar editor */
+  dialog = g_object_new (THUNAR_TYPE_TOOLBAR_EDITOR, NULL);
+
+  /* set the toolbar used for gathering information */
+  THUNAR_TOOLBAR_EDITOR (dialog)->toolbar = toolbar;
+
+  /* now that the toolbar is set, populate the tree view */
+  thunar_toolbar_editor_populate_model (THUNAR_TOOLBAR_EDITOR (dialog));
+
+  /* destroy the dialog */
+  gtk_widget_destroy (dialog);
+}
