@@ -29,7 +29,6 @@
 #include <string.h>
 #endif
 
-#include "thunar/thunar-file-monitor.h"
 #include "thunar/thunar-folder.h"
 #include "thunar/thunar-gio-extensions.h"
 #include "thunar/thunar-pango-extensions.h"
@@ -113,9 +112,6 @@ static void                 thunar_tree_model_sort                    (ThunarTre
                                                                        GNode                  *node);
 static gboolean             thunar_tree_model_cleanup_idle            (gpointer                user_data);
 static void                 thunar_tree_model_cleanup_idle_destroy    (gpointer                user_data);
-static void                 thunar_tree_model_file_changed            (ThunarFileMonitor      *file_monitor,
-                                                                       ThunarFile             *file,
-                                                                       ThunarTreeModel        *model);
 static void                 thunar_tree_model_device_added            (ThunarDeviceMonitor    *device_monitor,
                                                                        ThunarDevice           *device,
                                                                        ThunarTreeModel        *model);
@@ -137,10 +133,13 @@ static void                 thunar_tree_model_item_free               (ThunarTre
 static void                 thunar_tree_model_item_reset              (ThunarTreeModelItem    *item);
 static void                 thunar_tree_model_item_load_folder        (ThunarTreeModelItem    *item);
 static void                 thunar_tree_model_item_files_added        (ThunarTreeModelItem    *item,
-                                                                       GList                  *files,
+                                                                       GHashTable             *files,
                                                                        ThunarFolder           *folder);
 static void                 thunar_tree_model_item_files_removed      (ThunarTreeModelItem    *item,
-                                                                       GList                  *files,
+                                                                       GHashTable             *files,
+                                                                       ThunarFolder           *folder);
+static void                 thunar_tree_model_item_files_changed      (ThunarTreeModelItem    *item,
+                                                                       GHashTable             *files,
                                                                        ThunarFolder           *folder);
 static gboolean             thunar_tree_model_item_load_idle          (gpointer                user_data);
 static void                 thunar_tree_model_item_load_idle_destroy  (gpointer                user_data);
@@ -192,8 +191,6 @@ struct _ThunarTreeModel
   /* removable devices */
   ThunarDeviceMonitor        *device_monitor;
 
-  ThunarFileMonitor          *file_monitor;
-
   gboolean                    sort_case_sensitive;
 
   ThunarTreeModelVisibleFunc  visible_func;
@@ -286,7 +283,6 @@ thunar_tree_model_init (ThunarTreeModel *model)
 {
   ThunarTreeModelItem *item;
   ThunarFile          *file;
-  GFile               *home;
   GList               *system_paths = NULL;
   GList               *devices;
   GList               *lp;
@@ -303,10 +299,6 @@ thunar_tree_model_init (ThunarTreeModel *model)
   model->visible_data = NULL;
   model->cleanup_idle_id = 0;
 
-  /* connect to the file monitor */
-  model->file_monitor = thunar_file_monitor_get_default ();
-  g_signal_connect (G_OBJECT (model->file_monitor), "file-changed", G_CALLBACK (thunar_tree_model_file_changed), model);
-
   /* allocate the "virtual root node" */
   model->root = g_node_new (NULL);
 
@@ -326,8 +318,7 @@ thunar_tree_model_init (ThunarTreeModel *model)
     system_paths = g_list_append (system_paths, thunar_g_file_new_for_computer());
 
   /* add the home folder to the system paths */
-  home = thunar_g_file_new_for_home ();
-  system_paths = g_list_append (system_paths, g_object_ref (home));
+  system_paths = g_list_append (system_paths, thunar_g_file_new_for_home ());
 
   if (thunar_g_vfs_is_uri_scheme_supported ("recent"))
     system_paths = g_list_append (system_paths, thunar_g_file_new_for_recent());
@@ -350,10 +341,6 @@ thunar_tree_model_init (ThunarTreeModel *model)
       file = thunar_file_get (lp->data, NULL);
       if (G_LIKELY (file != NULL))
         {
-          /* watch the trash for changes */
-          if (thunar_file_is_trash (file))
-            thunar_file_watch (file);
-
           /* create and append the new node */
           item = thunar_tree_model_item_new_with_file (model, file);
           node = g_node_append_data (model->root, item);
@@ -371,13 +358,9 @@ thunar_tree_model_init (ThunarTreeModel *model)
           /* add the dummy node */
           g_node_append_data (node, NULL);
         }
-
-      /* release the system defined path */
-      g_object_unref (lp->data);
     }
 
-  g_list_free (system_paths);
-  g_object_unref (home);
+  g_list_free_full (system_paths, g_object_unref);
 
   /* setup the initial devices */
   devices = thunar_device_monitor_get_devices (model->device_monitor);
@@ -400,10 +383,6 @@ thunar_tree_model_finalize (GObject *object)
   if (model->cleanup_idle_id != 0)
     g_source_remove (model->cleanup_idle_id);
 
-  /* disconnect from the file monitor */
-  g_signal_handlers_disconnect_by_func (model->file_monitor, thunar_tree_model_file_changed, model);
-  g_object_unref (model->file_monitor);
-
   /* release all resources allocated to the model */
   g_node_traverse (model->root, G_POST_ORDER, G_TRAVERSE_ALL, -1, thunar_tree_model_node_traverse_free, NULL);
   g_node_destroy (model->root);
@@ -958,18 +937,24 @@ thunar_tree_model_cleanup_idle_destroy (gpointer user_data)
 
 
 static void
-thunar_tree_model_file_changed (ThunarFileMonitor *file_monitor,
-                                ThunarFile        *file,
-                                ThunarTreeModel   *model)
+thunar_tree_model_item_files_changed (ThunarTreeModelItem *item,
+                                      GHashTable          *files,
+                                      ThunarFolder        *folder)
 {
-  _thunar_return_if_fail (THUNAR_IS_FILE_MONITOR (file_monitor));
-  _thunar_return_if_fail (model->file_monitor == file_monitor);
-  _thunar_return_if_fail (THUNAR_IS_TREE_MODEL (model));
-  _thunar_return_if_fail (THUNAR_IS_FILE (file));
+  gpointer       key;
+  GHashTableIter iter;
+
+  _thunar_return_if_fail (THUNAR_IS_FOLDER (folder));
+  _thunar_return_if_fail (item->folder == folder);
 
   /* traverse the model and emit "row-changed" for the file's nodes */
-  if (thunar_file_is_directory (file))
-    g_node_traverse (model->root, G_PRE_ORDER, G_TRAVERSE_ALL, -1, thunar_tree_model_node_traverse_changed, file);
+  g_hash_table_iter_init (&iter, files);
+  while (g_hash_table_iter_next (&iter, &key, NULL))
+    {
+      ThunarFile *file = THUNAR_FILE (key);
+      if (thunar_file_is_directory (file))
+        g_node_traverse (item->model->root, G_PRE_ORDER, G_TRAVERSE_ALL, -1, thunar_tree_model_node_traverse_changed, file);
+    }
 }
 
 
@@ -1244,10 +1229,6 @@ thunar_tree_model_item_reset (ThunarTreeModelItem *item)
   /* disconnect from the file */
   if (G_LIKELY (item->file != NULL))
     {
-      /* unwatch the trash */
-      if (thunar_file_is_trash (item->file))
-        thunar_file_unwatch (item->file);
-
       /* release and reset the file */
       g_object_unref (G_OBJECT (item->file));
       item->file = NULL;
@@ -1273,23 +1254,33 @@ thunar_tree_model_item_load_folder (ThunarTreeModelItem *item)
 
 static void
 thunar_tree_model_item_files_added (ThunarTreeModelItem *item,
-                                    GList               *files,
+                                    GHashTable          *files,
                                     ThunarFolder        *folder)
 {
   ThunarTreeModel     *model = THUNAR_TREE_MODEL (item->model);
   ThunarFile          *file;
   GNode               *node = NULL;
-  GList               *lp;
+  GHashTableIter       iter;
+  gpointer             key;
 
   _thunar_return_if_fail (THUNAR_IS_FOLDER (folder));
   _thunar_return_if_fail (item->folder == folder);
   _thunar_return_if_fail (model->visible_func != NULL);
 
+  /* someone added files to trash ... let's just update the trash icon and return */
+  if (thunar_file_is_trash (thunar_folder_get_corresponding_file (folder)))
+    {
+      node = g_node_find (model->root, G_POST_ORDER, G_TRAVERSE_ALL, item);
+      thunar_tree_model_node_traverse_changed (node, thunar_folder_get_corresponding_file (folder));
+      return;
+    }
+
   /* process all specified files */
-  for (lp = files; lp != NULL; lp = lp->next)
+  g_hash_table_iter_init (&iter, files);
+  while (g_hash_table_iter_next (&iter, &key, NULL))
     {
       /* we don't care for anything except folders */
-      file = THUNAR_FILE (lp->data);
+      file = THUNAR_FILE (key);
       if (!thunar_file_is_directory (file))
         continue;
 
@@ -1319,7 +1310,7 @@ thunar_tree_model_item_files_added (ThunarTreeModelItem *item,
 
 static void
 thunar_tree_model_item_files_removed (ThunarTreeModelItem *item,
-                                      GList               *files,
+                                      GHashTable          *files,
                                       ThunarFolder        *folder)
 {
   ThunarTreeModel *model = item->model;
@@ -1327,8 +1318,9 @@ thunar_tree_model_item_files_removed (ThunarTreeModelItem *item,
   GtkTreeIter      iter;
   GNode           *child_node;
   GNode           *node;
-  GList           *lp;
   GSList          *inv_link;
+  GHashTableIter   file_iter;
+  gpointer         key;
 
   _thunar_return_if_fail (THUNAR_IS_FOLDER (folder));
   _thunar_return_if_fail (item->folder == folder);
@@ -1337,15 +1329,25 @@ thunar_tree_model_item_files_removed (ThunarTreeModelItem *item,
   node = g_node_find (model->root, G_POST_ORDER, G_TRAVERSE_ALL, item);
   _thunar_return_if_fail (node != NULL);
 
+  /* someone removed files from trash ..let's just update the trash icon */
+  if (thunar_file_is_trash (thunar_folder_get_corresponding_file (folder)))
+    {
+      thunar_tree_model_node_traverse_changed (node, thunar_folder_get_corresponding_file (folder));
+      return;
+    }
+
   /* check if the node has any visible children */
   if (G_LIKELY (node->children != NULL))
     {
       /* process all files */
-      for (lp = files; lp != NULL; lp = lp->next)
+      g_hash_table_iter_init (&file_iter, files);
+      while (g_hash_table_iter_next (&file_iter, &key, NULL))
         {
+          ThunarFile *file = THUNAR_FILE (key);
+
           /* find the child node for the file */
           for (child_node = g_node_first_child (node); child_node != NULL; child_node = g_node_next_sibling (child_node))
-            if (child_node->data != NULL && THUNAR_TREE_MODEL_ITEM (child_node->data)->file == lp->data)
+            if (child_node->data != NULL && THUNAR_TREE_MODEL_ITEM (child_node->data)->file == file)
               break;
 
           /* drop the child node (and all descendant nodes) from the model */
@@ -1369,14 +1371,17 @@ thunar_tree_model_item_files_removed (ThunarTreeModelItem *item,
   /* we also need to release all the invisible folders */
   if (item->invisible_children != NULL)
     {
-      for (lp = files; lp != NULL; lp = lp->next)
+      g_hash_table_iter_init (&file_iter, files);
+      while (g_hash_table_iter_next (&file_iter, &key, NULL))
         {
+          ThunarFile *file = THUNAR_FILE (key);
+
           /* find the file in the hidden list */
-          inv_link = g_slist_find (item->invisible_children, lp->data);
+          inv_link = g_slist_find (item->invisible_children, file);
           if (inv_link != NULL)
             {
               /* release the file */
-              g_object_unref (G_OBJECT (lp->data));
+              g_object_unref (G_OBJECT (file));
 
               /* remove from the list */
               item->invisible_children = g_slist_delete_link (item->invisible_children, inv_link);
@@ -1418,7 +1423,7 @@ thunar_tree_model_item_load_idle (gpointer user_data)
 {
   ThunarTreeModelItem *item = user_data;
   GFile               *mount_point;
-  GList               *files;
+  GHashTable          *files;
 #ifndef NDEBUG
   GNode               *node;
 #endif
@@ -1448,7 +1453,7 @@ thunar_tree_model_item_load_idle (gpointer user_data)
     }
 
   /* verify that we have a file */
-  if (G_LIKELY (item->file != NULL))
+  if (G_LIKELY (item->file != NULL) && item->folder == NULL)
     {
       /* open the folder for the item */
       item->folder = thunar_folder_get_for_file (item->file);
@@ -1457,19 +1462,17 @@ thunar_tree_model_item_load_idle (gpointer user_data)
           /* connect signals */
           g_signal_connect_swapped (G_OBJECT (item->folder), "files-added", G_CALLBACK (thunar_tree_model_item_files_added), item);
           g_signal_connect_swapped (G_OBJECT (item->folder), "files-removed", G_CALLBACK (thunar_tree_model_item_files_removed), item);
+          g_signal_connect_swapped (G_OBJECT (item->folder), "files-changed", G_CALLBACK (thunar_tree_model_item_files_changed), item);
           g_signal_connect_swapped (G_OBJECT (item->folder), "notify::loading", G_CALLBACK (thunar_tree_model_item_notify_loading), item);
 
+          /* If the folder is already loaded, directly update the loading state */
+          if (!thunar_folder_get_loading (item->folder))
+            thunar_tree_model_item_notify_loading (item, NULL, item->folder);
+
           /* load the initial set of files (if any) */
           files = thunar_folder_get_files (item->folder);
           if (G_UNLIKELY (files != NULL))
-            {
-              thunar_tree_model_item_files_added (item, files, item->folder);
-              g_list_free (files);
-            }
-
-          /* notify for "loading" if already loaded */
-          if (!thunar_folder_get_loading (item->folder))
-            g_object_notify (G_OBJECT (item->folder), "loading");
+            thunar_tree_model_item_files_added (item, files, item->folder);
         }
     }
 
