@@ -56,7 +56,6 @@
 
 #include "thunar/thunar-application.h"
 #include "thunar/thunar-file.h"
-#include "thunar/thunar-file-monitor.h"
 #include "thunar/thunar-gobject-extensions.h"
 #include "thunar/thunar-tree-view-model.h"
 #include "thunar/thunar-preferences.h"
@@ -64,7 +63,6 @@
 #include "thunar/thunar-user.h"
 #include "thunar/thunar-simple-job.h"
 #include "thunar/thunar-util.h"
-#include "thunar/thunar-file-monitor.h"
 #include "thunar/thunar-io-jobs.h"
 #include "thunar/thunar-gio-extensions.h"
 
@@ -244,7 +242,7 @@ static void              thunar_tree_view_model_node_add_child (Node *node,
                                                                 Node *child);
 static void              thunar_tree_view_model_node_add_dummy_child (Node *node);
 static void              thunar_tree_view_model_node_drop_dummy_child (Node *node);
-static void              thunar_tree_view_model_dir_add_file (Node       *node,
+static Node             *thunar_tree_view_model_dir_add_file (Node       *node,
                                                               ThunarFile *file);
 static void              thunar_tree_view_model_dir_remove_file (Node       *node,
                                                                  ThunarFile *file);
@@ -256,12 +254,11 @@ static gint              thunar_tree_view_model_cmp_nodes (gconstpointer a,
 static void              thunar_tree_view_model_sort (ThunarTreeViewModel *model);
 static void              thunar_tree_view_model_load_dir (Node *node);
 static void              thunar_tree_view_model_cleanup_model (ThunarTreeViewModel *model);
-static void              thunar_tree_view_model_file_count_callback (ExoJob  *job,
-                                                                     gpointer model);
+static void              thunar_tree_view_model_file_count_callback (ExoJob              *job,
+                                                                     ThunarTreeViewModel *model);
 static void              thunar_tree_view_model_node_destroy (Node *node);
-static void              thunar_tree_view_model_file_changed (ThunarFileMonitor   *monitor,
-                                                              ThunarFile          *file,
-                                                              ThunarTreeViewModel *model);
+static void              thunar_tree_view_model_dir_files_changed (Node          *node,
+                                                                   GHashTable    *files);
 static void              thunar_tree_view_model_set_loading (ThunarTreeViewModel *model,
                                                              gboolean             loading);
 static gboolean          thunar_tree_view_model_update_search_files (ThunarTreeViewModel *model);
@@ -299,8 +296,6 @@ struct _ThunarTreeViewModel
   Node                 *root;
   ThunarFolder         *dir;
 
-  ThunarFileMonitor    *monitor;
-
   /* Hashtable for quick access to all directories and sub-directories which are managed by this view
    * The key is the corresponding #ThunarFile of the directory, and the value is the #_Node of the directory */
   GHashTable           *subdirs;
@@ -343,7 +338,8 @@ struct _Node
   gint                 n_children;
   gboolean             loaded;
   gboolean             expanded;
-
+  gboolean             file_watch_active;
+ 
   /* set of all the children gfiles;
    * contains mappings of (gfile -> GSequenceIter *(_child_node->ptr)) */
   GHashTable          *set;
@@ -365,7 +361,6 @@ G_DEFINE_TYPE_WITH_CODE (ThunarTreeViewModel, thunar_tree_view_model, G_TYPE_OBJ
 
 
 /* Reference to GParamSpec(s) of all the N_PROPERTIES */
-static guint       tree_model_signals[THUNAR_STANDARD_VIEW_MODEL_LAST_SIGNAL];
 static GParamSpec *tree_model_props[N_PROPERTIES] = {
   NULL,
 };
@@ -477,8 +472,6 @@ thunar_tree_view_model_class_init (ThunarTreeViewModelClass *klass)
 
   /* install properties */
   g_object_class_install_properties (gobject_class, N_PROPERTIES, tree_model_props);
-
-  /* No need to install signals. Already done by the interface */
 }
 
 
@@ -502,10 +495,6 @@ thunar_tree_view_model_init (ThunarTreeViewModel *model)
   model->loading = 0;
 
   model->subdirs = g_hash_table_new (g_direct_hash, g_direct_equal);
-
-  model->monitor = thunar_file_monitor_get_default ();
-  g_signal_connect (G_OBJECT (model->monitor), "file-changed",
-                    G_CALLBACK (thunar_tree_view_model_file_changed), model);
 }
 
 
@@ -595,11 +584,6 @@ thunar_tree_view_model_finalize (GObject *object)
 
   g_mutex_clear (&model->mutex_add_search_files);
 
-  /* disconnect from the file monitor */
-  g_signal_handlers_disconnect_by_func (G_OBJECT (model->monitor),
-                                        thunar_tree_view_model_file_changed, model);
-  g_object_unref (G_OBJECT (model->monitor));
-
   g_free (model->date_custom_style);
   g_strfreev (model->search_terms);
 
@@ -1052,7 +1036,7 @@ thunar_tree_view_model_get_path (GtkTreeModel *model,
   node = g_sequence_get (iter->user_data);
 
   depth = node->depth;
-  indices = g_malloc_n (sizeof (gint), depth);
+  indices = g_malloc_n (depth, sizeof (gint));
 
   for (gint d = depth - 1; d >= 0; --d)
     {
@@ -1080,7 +1064,7 @@ thunar_tree_view_model_get_value (GtkTreeModel *model,
   const gchar  *real_name;
   ThunarUser   *user = NULL;
   ThunarFolder *folder;
-  guint32       item_count;
+  gint32        item_count;
   GFile        *g_file;
   GFile        *g_file_parent = NULL;
   gchar        *str = NULL;
@@ -1100,7 +1084,7 @@ thunar_tree_view_model_get_value (GtkTreeModel *model,
         g_value_init (value, G_TYPE_STRING);
         if (file == NULL)
           {
-            g_value_set_static_string (value, _ (""));
+            g_value_set_static_string (value, "");
             break;
           }
         str = thunar_file_get_date_string (file, THUNAR_FILE_DATE_CREATED, THUNAR_TREE_VIEW_MODEL (model)->date_style, THUNAR_TREE_VIEW_MODEL (model)->date_custom_style);
@@ -1111,7 +1095,7 @@ thunar_tree_view_model_get_value (GtkTreeModel *model,
         g_value_init (value, G_TYPE_STRING);
         if (file == NULL)
           {
-            g_value_set_static_string (value, _ (""));
+            g_value_set_static_string (value, "");
             break;
           }
         str = thunar_file_get_date_string (file, THUNAR_FILE_DATE_ACCESSED, THUNAR_TREE_VIEW_MODEL (model)->date_style, THUNAR_TREE_VIEW_MODEL (model)->date_custom_style);
@@ -1122,7 +1106,7 @@ thunar_tree_view_model_get_value (GtkTreeModel *model,
         g_value_init (value, G_TYPE_STRING);
         if (file == NULL)
           {
-            g_value_set_static_string (value, _ (""));
+            g_value_set_static_string (value, "");
             break;
           }
         str = thunar_file_get_date_string (file, THUNAR_FILE_DATE_MODIFIED, THUNAR_TREE_VIEW_MODEL (model)->date_style, THUNAR_TREE_VIEW_MODEL (model)->date_custom_style);
@@ -1133,7 +1117,7 @@ thunar_tree_view_model_get_value (GtkTreeModel *model,
         g_value_init (value, G_TYPE_STRING);
         if (file == NULL)
           {
-            g_value_set_static_string (value, _ (""));
+            g_value_set_static_string (value, "");
             break;
           }
         str = thunar_file_get_date_string (file, THUNAR_FILE_DATE_DELETED, THUNAR_TREE_VIEW_MODEL (model)->date_style, THUNAR_TREE_VIEW_MODEL (model)->date_custom_style);
@@ -1144,7 +1128,7 @@ thunar_tree_view_model_get_value (GtkTreeModel *model,
         g_value_init (value, G_TYPE_STRING);
         if (file == NULL)
           {
-            g_value_set_static_string (value, _ (""));
+            g_value_set_static_string (value, "");
             break;
           }
         str = thunar_file_get_date_string (file, THUNAR_FILE_RECENCY, THUNAR_TREE_VIEW_MODEL (model)->date_style, THUNAR_TREE_VIEW_MODEL (model)->date_custom_style);
@@ -1213,7 +1197,7 @@ thunar_tree_view_model_get_value (GtkTreeModel *model,
           }
         else
           {
-            g_value_set_static_string (value, _ (""));
+            g_value_set_static_string (value, "");
           }
         break;
 
@@ -1221,7 +1205,7 @@ thunar_tree_view_model_get_value (GtkTreeModel *model,
         g_value_init (value, G_TYPE_STRING);
         if (file == NULL)
           {
-            g_value_set_static_string (value, _ (""));
+            g_value_set_static_string (value, "");
             break;
           }
         g_value_set_static_string (value, thunar_file_get_content_type (file));
@@ -1260,7 +1244,7 @@ thunar_tree_view_model_get_value (GtkTreeModel *model,
           }
         else
           {
-            g_value_set_static_string (value, _ (""));
+            g_value_set_static_string (value, "");
           }
         break;
 
@@ -1268,7 +1252,7 @@ thunar_tree_view_model_get_value (GtkTreeModel *model,
         g_value_init (value, G_TYPE_STRING);
         if (file == NULL)
           {
-            g_value_set_static_string (value, _ (""));
+            g_value_set_static_string (value, "");
             break;
           }
         g_value_take_string (value, thunar_file_get_mode_string (file));
@@ -1278,7 +1262,7 @@ thunar_tree_view_model_get_value (GtkTreeModel *model,
         g_value_init (value, G_TYPE_STRING);
         if (file == NULL)
           {
-            g_value_set_static_string (value, _ (""));
+            g_value_set_static_string (value, "");
             break;
           }
         if (thunar_file_is_mountable (file))
@@ -1300,7 +1284,10 @@ thunar_tree_view_model_get_value (GtkTreeModel *model,
             else if (THUNAR_TREE_VIEW_MODEL (model)->folder_item_count == THUNAR_FOLDER_ITEM_COUNT_ALWAYS)
               {
                 item_count = thunar_file_get_file_count (file, G_CALLBACK (thunar_tree_view_model_file_count_callback), model);
-                g_value_take_string (value, g_strdup_printf (ngettext ("%u item", "%u items", item_count), item_count));
+                if (item_count < 0)
+                  g_value_take_string (value, g_strdup (_("unknown")));
+                else
+                  g_value_take_string (value, g_strdup_printf (ngettext ("%u item", "%u items", item_count), item_count));
               }
 
             /* If the option is set to always show folder sizes as item counts only for local files,
@@ -1310,7 +1297,10 @@ thunar_tree_view_model_get_value (GtkTreeModel *model,
                 if (thunar_file_is_local (file))
                   {
                     item_count = thunar_file_get_file_count (file, G_CALLBACK (thunar_tree_view_model_file_count_callback), model);
-                    g_value_take_string (value, g_strdup_printf (ngettext ("%u item", "%u items", item_count), item_count));
+                    if (item_count < 0)
+                      g_value_take_string (value, g_strdup (_("unknown")));
+                    else
+                      g_value_take_string (value, g_strdup_printf (ngettext ("%u item", "%u items", item_count), item_count));
                   }
                 else
                   g_value_take_string (value, thunar_file_get_size_string_formatted (file, THUNAR_TREE_VIEW_MODEL (model)->file_size_binary));
@@ -1328,7 +1318,7 @@ thunar_tree_view_model_get_value (GtkTreeModel *model,
         g_value_init (value, G_TYPE_STRING);
         if (file == NULL)
           {
-            g_value_set_static_string (value, _ (""));
+            g_value_set_static_string (value, "");
             break;
           }
         g_value_take_string (value, thunar_file_get_size_in_bytes_string (file));
@@ -1338,7 +1328,7 @@ thunar_tree_view_model_get_value (GtkTreeModel *model,
         g_value_init (value, G_TYPE_STRING);
         if (file == NULL)
           {
-            g_value_set_static_string (value, _ (""));
+            g_value_set_static_string (value, "");
             break;
           }
         device_type = thunar_file_get_device_type (file);
@@ -1502,7 +1492,7 @@ thunar_tree_view_model_iter_nth_child (GtkTreeModel *model,
   else
     node = g_sequence_get (parent->user_data);
 
-  if (n >= node->n_children)
+  if (node == NULL || n >= node->n_children)
     return FALSE;
 
   ptr = g_sequence_get_iter_at_pos (node->children, n);
@@ -1616,6 +1606,7 @@ thunar_tree_view_model_get_job (ThunarStandardViewModel *model)
 
 
 
+G_NORETURN
 static void
 _thunar_tree_view_model_search_error (ThunarJob *job)
 {
@@ -2053,6 +2044,8 @@ thunar_tree_view_model_new_node (ThunarFile *file)
 
   _node->scheduled_unload_id = 0;
 
+  _node->file_watch_active = FALSE;
+
   return _node;
 }
 
@@ -2221,7 +2214,7 @@ thunar_tree_view_model_node_drop_dummy_child (Node *node)
 
 
 
-static void
+static Node*
 thunar_tree_view_model_dir_add_file (Node       *node,
                                      ThunarFile *file)
 {
@@ -2229,13 +2222,17 @@ thunar_tree_view_model_dir_add_file (Node       *node,
   GtkTreePath *path;
   Node        *child;
 
-  THUNAR_WARN_VOID_RETURN (g_hash_table_contains (node->set, file));
+  if (g_hash_table_contains (node->set, file))
+    {
+      GSequenceIter *iter = g_hash_table_lookup (node->set, file);
+      return g_sequence_get (iter);
+    }
 
   child = thunar_tree_view_model_new_node (file);
   thunar_tree_view_model_node_add_child (node, child);
 
   if (thunar_file_is_directory (file)
-      && !thunar_g_file_is_empty (thunar_file_get_file (file)))
+      && !thunar_file_is_empty_directory (file))
     thunar_tree_view_model_node_add_dummy_child (child);
 
   /* notify the model if a child has been added to previously empty folder */
@@ -2246,6 +2243,8 @@ thunar_tree_view_model_dir_add_file (Node       *node,
       gtk_tree_model_row_has_child_toggled (GTK_TREE_MODEL (node->model), path, &tree_iter);
       gtk_tree_path_free (path);
     }
+
+  return child;
 }
 
 
@@ -2306,9 +2305,15 @@ thunar_tree_view_model_locate_file (ThunarTreeViewModel *model,
 
   parent = thunar_file_get_parent (file, NULL);
 
-  if (parent == NULL || !g_hash_table_contains (model->subdirs, parent))
+  if (parent == NULL)
     return NULL;
 
+  if (!g_hash_table_contains (model->subdirs, parent))
+    {
+      g_object_unref (parent);
+      return NULL;
+    }
+
   parent_node = g_hash_table_lookup (model->subdirs, parent);
   g_object_unref (parent);
   THUNAR_WARN_RETURN_VAL (parent_node == NULL, NULL);
@@ -2324,33 +2329,6 @@ thunar_tree_view_model_locate_file (ThunarTreeViewModel *model,
 
 
 
-static Node *
-thunar_tree_view_model_locate_parent (ThunarTreeViewModel *model,
-                                      ThunarFile          *file)
-{
-  ThunarFile    *parent;
-  Node          *parent_node;
-
-  _thunar_return_val_if_fail (THUNAR_IS_TREE_VIEW_MODEL (model), NULL);
-  _thunar_return_val_if_fail (THUNAR_IS_FILE (file), NULL);
-
-  if (!thunar_file_has_parent (file))
-    return NULL;
-
-  parent = thunar_file_get_parent (file, NULL);
-
-  if (parent == NULL || !g_hash_table_contains (model->subdirs, parent))
-    return NULL;
-
-  parent_node = g_hash_table_lookup (model->subdirs, parent);
-  g_object_unref (parent);
-  THUNAR_WARN_RETURN_VAL (parent_node == NULL, NULL);
-
-  return parent_node;
-}
-
-
-
 static gint
 thunar_tree_view_model_cmp_nodes (gconstpointer a,
                                   gconstpointer b,
@@ -2500,23 +2478,22 @@ _thunar_tree_view_model_folder_error (Node         *node,
       else
         thunar_tree_view_model_dir_remove_file (node->parent, node->file);
     }
-
-  /* forward the error signal */
-  g_signal_emit (G_OBJECT (node->model), tree_model_signals[THUNAR_STANDARD_VIEW_MODEL_ERROR], 0, error);
 }
 
 
 
 static void
-_thunar_tree_view_model_dir_files_added (Node  *node,
-                                         GList *files)
+_thunar_tree_view_model_dir_files_added (Node       *node,
+                                         GHashTable *files)
 {
-  ThunarFile *file;
-  GList      *lp;
+  ThunarFile    *file;
+  GHashTableIter iter;
+  gpointer       key;
 
-  for (lp = files; lp != NULL; lp = lp->next)
+  g_hash_table_iter_init (&iter, files);
+  while (g_hash_table_iter_next (&iter, &key, NULL))
     {
-      file = THUNAR_FILE (lp->data);
+      file = THUNAR_FILE (key);
 
       if (thunar_file_is_hidden (file) && !g_hash_table_contains (node->hidden_files, file))
         {
@@ -2526,22 +2503,25 @@ _thunar_tree_view_model_dir_files_added (Node  *node,
             continue;
         }
 
-      thunar_tree_view_model_dir_add_file (node, file);
+      if (!g_hash_table_contains (node->set, file))
+        thunar_tree_view_model_dir_add_file (node, file);
     }
 }
 
 
 
 static void
-_thunar_tree_view_model_dir_files_removed (Node  *node,
-                                           GList *files)
+_thunar_tree_view_model_dir_files_removed (Node       *node,
+                                           GHashTable *files)
 {
-  ThunarFile *file;
-  GList      *lp;
+  ThunarFile    *file;
+  GHashTableIter iter;
+  gpointer       key;
 
-  for (lp = files; lp != NULL; lp = lp->next)
+  g_hash_table_iter_init (&iter, files);
+  while (g_hash_table_iter_next (&iter, &key, NULL))
     {
-      file = THUNAR_FILE (lp->data);
+      file = THUNAR_FILE (key);
 
       /* we cannot trust thunar_file_is_hidden here;
        * don't know why. Maybe the file has gone through dispose */
@@ -2581,7 +2561,7 @@ _thunar_tree_view_model_dir_notify_loading (Node         *node,
 static void
 thunar_tree_view_model_load_dir (Node *node)
 {
-  GList *files;
+  GHashTable *files;
 
   _thunar_return_if_fail (node != NULL);
 
@@ -2600,17 +2580,16 @@ thunar_tree_view_model_load_dir (Node *node)
   g_signal_connect_swapped (G_OBJECT (node->dir), "error", G_CALLBACK (_thunar_tree_view_model_folder_error), node);
   g_signal_connect_swapped (G_OBJECT (node->dir), "files-added", G_CALLBACK (_thunar_tree_view_model_dir_files_added), node);
   g_signal_connect_swapped (G_OBJECT (node->dir), "files-removed", G_CALLBACK (_thunar_tree_view_model_dir_files_removed), node);
+  g_signal_connect_swapped (G_OBJECT (node->dir), "files-changed", G_CALLBACK (thunar_tree_view_model_dir_files_changed), node);
   g_signal_connect_swapped (G_OBJECT (node->dir), "notify::loading", G_CALLBACK (_thunar_tree_view_model_dir_notify_loading), node);
 
   files = thunar_folder_get_files (node->dir);
   if (files != NULL)
-    {
-      _thunar_tree_view_model_dir_files_added (node, files);
-      g_list_free (files);
-    }
+    _thunar_tree_view_model_dir_files_added (node, files);
 
+  /* If the folder is already loaded, directly update the loading state */
   if (!thunar_folder_get_loading (node->dir))
-    g_object_notify (G_OBJECT (node->dir), "loading");
+    _thunar_tree_view_model_dir_notify_loading (node, NULL, node->dir);
 
   g_hash_table_insert (node->model->subdirs, node->file, node);
 }
@@ -2663,11 +2642,14 @@ thunar_tree_view_model_cleanup_model (ThunarTreeViewModel *model)
 
 
 static void
-thunar_tree_view_model_file_count_callback (ExoJob  *job,
-                                            gpointer model)
+thunar_tree_view_model_file_count_callback (ExoJob              *job,
+                                            ThunarTreeViewModel *model)
 {
-  GArray     *param_values;
-  ThunarFile *file;
+  GArray              *param_values;
+  ThunarFile          *file;
+  ThunarFile          *parent;
+  GHashTable          *files = g_hash_table_new (g_direct_hash, NULL);
+  Node                *parent_node;
 
   if (job == NULL)
     return;
@@ -2678,7 +2660,19 @@ thunar_tree_view_model_file_count_callback (ExoJob  *job,
   if (file == NULL)
     return;
 
-  thunar_tree_view_model_file_changed (NULL, file, THUNAR_TREE_VIEW_MODEL (model));
+  parent = thunar_file_get_parent (file, NULL);
+  if (parent == NULL)
+    return;
+
+  parent_node = thunar_tree_view_model_locate_file (model, parent);
+  g_object_unref (parent);
+
+   if (parent_node == NULL)
+     return;
+
+   g_hash_table_add (files, file);
+   thunar_tree_view_model_dir_files_changed (parent_node, files);
+   g_hash_table_destroy (files);
 }
 
 
@@ -2688,6 +2682,13 @@ thunar_tree_view_model_node_destroy (Node *node)
 {
   GSequenceIter *iter;
 
+  /* stop possible file-watch */
+  if (node->file != NULL && node->file_watch_active)
+    {
+      g_signal_handlers_disconnect_by_data (node->file, node);
+      thunar_file_unwatch (node->file);
+    }
+
   if (node->scheduled_unload_id != 0)
     {
       g_source_remove (node->scheduled_unload_id);
@@ -2732,121 +2733,110 @@ thunar_tree_view_model_node_destroy (Node *node)
 
 
 static void
-thunar_tree_view_model_file_changed (ThunarFileMonitor   *monitor,
-                                     ThunarFile          *file,
-                                     ThunarTreeViewModel *model)
+thunar_tree_view_model_dir_files_changed (Node       *node_parent,
+                                          GHashTable *files)
 {
+  ThunarTreeViewModel *model = node_parent->model;
   GSequenceIter *iter;
   GtkTreeIter    tree_iter;
+  GHashTableIter files_iter;
   GtkTreePath   *path;
-  Node          *node, *parent;
-  gint           pos_after, pos_before = 0;
+  Node          *node;
+  gint           pos_after, pos_before;
   gint          *new_order;
   gint           length;
-  gboolean       dummy_added;
+  gboolean       dummy_added = FALSE;
+  ThunarFile    *file;
+  gpointer       key;
 
-  node = thunar_tree_view_model_locate_file (model, file);
-
-  if (node == NULL)
+  g_hash_table_iter_init (&files_iter, files);
+  while (g_hash_table_iter_next (&files_iter, &key, NULL))
     {
-      parent = thunar_tree_view_model_locate_parent (model, file);
-
-      if (parent == NULL)
-        return;
+      file = THUNAR_FILE (key);
 
       /* two cases - file is hidden or not
-       * 1. if it is in hidden list but not hidden anymore then add the new file
-       * 2. if it is not hidden but has turned hidden hide it */
-      if (g_hash_table_contains (parent->hidden_files, file) && !thunar_file_is_hidden (file))
+      * 1. if it is in hidden list but not hidden anymore then add the new file
+      * 2. if it is not hidden but has turned hidden hide it */
+      if (g_hash_table_contains (node_parent->hidden_files, file) && !thunar_file_is_hidden (file))
         {
-          g_hash_table_remove (parent->hidden_files, file);
-          thunar_tree_view_model_dir_add_file (parent, file);
-          return;
+          g_hash_table_remove (node_parent->hidden_files, file);
+          thunar_tree_view_model_dir_add_file (node_parent, file);
+          continue;
         }
 
       /* file is now hidden but still in the visible list */
-      if (g_hash_table_contains (parent->set, file) && thunar_file_is_hidden (file))
+      if (g_hash_table_contains (node_parent->set, file) && thunar_file_is_hidden (file))
         {
           if (!model->show_hidden)
-            thunar_tree_view_model_dir_remove_file (parent, file);
-          g_hash_table_add (parent->hidden_files, g_object_ref (file));
-          return;
+            thunar_tree_view_model_dir_remove_file (node_parent, file);
+          g_hash_table_add (node_parent->hidden_files, g_object_ref (file));
+          continue;
         }
 
-      return;
-    }
+      node = thunar_tree_view_model_locate_file (model, file);
+      /* we dont have the file */
+      if (node == NULL)
+        continue;
 
-  if (thunar_file_is_hidden (file))
-    {
-      /* when a file has been renamed to a hidden file then we need to check
-       * if a file exists in the parent's hidden file list or not & add it if
-       * not there */
-      if (!g_hash_table_contains (node->parent->hidden_files, file))
-        g_hash_table_add (node->parent->hidden_files, file);
-      /* if show_hidden is inactive and a normal file has been renamed
-       * to hidden file then hide that file */
-      if (!model->show_hidden && g_hash_table_contains (node->parent->set, file))
-        return thunar_tree_view_model_dir_remove_file (node->parent, file);
-    }
+      iter = node->ptr;
 
-  iter = node->ptr;
+      pos_before = g_sequence_iter_get_position (iter);
+      g_sequence_sort_changed (iter, thunar_tree_view_model_cmp_nodes, model);
+      pos_after = g_sequence_iter_get_position (iter);
 
-  pos_before = g_sequence_iter_get_position (iter);
-  g_sequence_sort_changed (iter, thunar_tree_view_model_cmp_nodes, model);
-  pos_after = g_sequence_iter_get_position (iter);
-
-  if (pos_before != pos_after)
-    {
-      length = node->parent->n_children;
-      new_order = g_malloc_n (sizeof (gint), length);
-
-      /* new_order[newpos] = oldpos */
-      for (gint i = 0, j = 0; i < length; ++i)
+      if (pos_before != pos_after)
         {
-          if (G_UNLIKELY (i == pos_after))
+          length = node_parent->n_children;
+          new_order = g_malloc_n (length, sizeof (gint));
+
+          /* new_order[newpos] = oldpos */
+          for (gint i = 0, j = 0; i < length; ++i)
             {
-              new_order[i] = pos_before;
+              if (G_UNLIKELY (i == pos_after))
+                {
+                  new_order[i] = pos_before;
+                }
+              else
+                {
+                  if (G_UNLIKELY (j == pos_before))
+                    j++;
+                  new_order[i] = j++;
+                }
+            }
+
+          if (node_parent->ptr != NULL)
+            {
+              GTK_TREE_ITER_INIT (tree_iter, model->stamp, node_parent->ptr);
+              path = gtk_tree_model_get_path (GTK_TREE_MODEL (model), &tree_iter);
+              gtk_tree_model_rows_reordered (GTK_TREE_MODEL (model), path, &tree_iter, new_order);
+              gtk_tree_path_free (path);
             }
           else
             {
-              if (G_UNLIKELY (j == pos_before))
-                j++;
-              new_order[i] = j++;
+              path = gtk_tree_path_new_first ();
+              gtk_tree_model_rows_reordered (GTK_TREE_MODEL (model), path, NULL, new_order);
+              gtk_tree_path_free (path);
             }
+
+          g_free (new_order);
         }
 
-      if (node->parent->ptr != NULL)
+      if (thunar_file_is_directory (file)
+          && !thunar_file_is_empty_directory (file)
+          && !node->loaded
+          && !thunar_tree_view_model_node_has_dummy_child (node))
         {
-          GTK_TREE_ITER_INIT (tree_iter, model->stamp, node->parent->ptr);
-          path = gtk_tree_model_get_path (GTK_TREE_MODEL (model), &tree_iter);
-          gtk_tree_model_rows_reordered (GTK_TREE_MODEL (model), path, &tree_iter, new_order);
-          gtk_tree_path_free (path);
-        }
-      else
-        {
-          path = gtk_tree_path_new_first ();
-          gtk_tree_model_rows_reordered (GTK_TREE_MODEL (model), path, NULL, new_order);
-          gtk_tree_path_free (path);
+          dummy_added = TRUE;
+          thunar_tree_view_model_node_add_dummy_child (node);
         }
 
-      g_free (new_order);
-    }
-
-  if (thunar_file_is_directory (file)
-      && !thunar_g_file_is_empty (thunar_file_get_file (file))
-      && !node->loaded
-      && !thunar_tree_view_model_node_has_dummy_child (node))
-    {
-      dummy_added = TRUE;
-      thunar_tree_view_model_node_add_dummy_child (node);
-    }
-
-  GTK_TREE_ITER_INIT (tree_iter, model->stamp, iter);
-  path = gtk_tree_model_get_path (GTK_TREE_MODEL (model), &tree_iter);
-  gtk_tree_model_row_changed (GTK_TREE_MODEL (model), path, &tree_iter);
-  if (dummy_added)
-    gtk_tree_model_row_has_child_toggled (GTK_TREE_MODEL (model), path, &tree_iter);
-  gtk_tree_path_free (path);
+      GTK_TREE_ITER_INIT (tree_iter, model->stamp, iter);
+      path = gtk_tree_model_get_path (GTK_TREE_MODEL (model), &tree_iter);
+      gtk_tree_model_row_changed (GTK_TREE_MODEL (model), path, &tree_iter);
+      if (dummy_added)
+        gtk_tree_model_row_has_child_toggled (GTK_TREE_MODEL (model), path, &tree_iter);
+      gtk_tree_path_free (path);
+  } /* end for all files */
 }
 
 
@@ -2967,28 +2957,87 @@ thunar_tree_view_model_schedule_unload (ThunarTreeViewModel *model,
 
 
 
+static gboolean
+_thunar_tree_view_model_matches_search_terms (ThunarTreeViewModel *model,
+                                             ThunarFile           *file)
+{
+  gboolean    matched;
+  gchar      *name_n;
+
+  name_n = (gchar *) thunar_file_get_display_name (file);
+  if (name_n == NULL)
+    return FALSE;
+
+  name_n = thunar_g_utf8_normalize_for_search (name_n, TRUE, TRUE);
+  matched = thunar_util_search_terms_match (model->search_terms, name_n);
+  g_free (name_n);
+
+  return matched;
+}
+
+
+
+static void
+_thunar_tree_view_model_search_file_destroyed (Node *node, ThunarFile *file)
+{
+  GHashTable    *files = g_hash_table_new (g_direct_hash, NULL);
+
+  g_hash_table_add (files, file);
+  _thunar_tree_view_model_dir_files_removed (node->model->root, files);
+  g_hash_table_destroy (files);
+}
+
+
+
+static void
+_thunar_tree_view_model_search_file_changed (Node *node, ThunarFile *file)
+{
+  GHashTable *files = g_hash_table_new (g_direct_hash, NULL);
+
+  g_hash_table_add (files, file);
+
+  if (_thunar_tree_view_model_matches_search_terms (node->model, node->file) == TRUE)
+    thunar_tree_view_model_dir_files_changed (node->model->root, files);
+  else
+    _thunar_tree_view_model_dir_files_removed (node->model->root, files);
+
+  g_hash_table_destroy (files);
+}
+
+
+
 static gboolean
 thunar_tree_view_model_update_search_files (ThunarTreeViewModel *model)
 {
   ThunarFile *file;
-  gboolean    matched;
-  gchar      *name_n;
 
   g_mutex_lock (&model->mutex_add_search_files);
 
   for (GList *lp = model->search_files; lp != NULL; lp = lp->next)
     {
+      file = THUNAR_FILE (lp->data);
+      if (THUNAR_IS_FILE (file) == FALSE)
+        {
+          g_warning ("failed to add file to search results");
+          continue;
+        }
+
       /* take a reference on that file */
-      file = THUNAR_FILE (g_object_ref (G_OBJECT (lp->data)));
-      _thunar_return_val_if_fail (THUNAR_IS_FILE (file), TRUE);
+      g_object_ref (file);
 
-      name_n = (gchar *) thunar_file_get_display_name (file);
-      name_n = thunar_g_utf8_normalize_for_search (name_n, TRUE, TRUE);
-      matched = thunar_util_search_terms_match (model->search_terms, name_n);
-      g_free (name_n);
-
-      if (matched)
-        thunar_tree_view_model_dir_add_file (model->root, lp->data);
+      if (_thunar_tree_view_model_matches_search_terms (model, file))
+      {
+        Node *node;
+        node = thunar_tree_view_model_dir_add_file (model->root, lp->data);
+        if (node != NULL)
+          {
+            /* start watching the file */
+            g_signal_connect_swapped (file, "destroy", G_CALLBACK (_thunar_tree_view_model_search_file_destroyed), node);
+            g_signal_connect_swapped (file, "changed", G_CALLBACK (_thunar_tree_view_model_search_file_changed), node);
+            thunar_file_watch (file);
+            node->file_watch_active = TRUE;
+          }
+      }
 
       g_object_unref (file);
     }
@@ -3014,4 +3063,4 @@ thunar_tree_view_model_add_search_files (ThunarStandardViewModel *model,
   _model->search_files = g_list_concat (_model->search_files, files);
 
   g_mutex_unlock (&_model->mutex_add_search_files);
-}
\ No newline at end of file
+}
