@@ -344,11 +344,8 @@ thunar_tree_view_init (ThunarTreeView *view)
   /* grab a reference on the provider factory */
   view->provider_factory = thunarx_provider_factory_get_default ();
 
-  /* grab a reference on the preferences; be sure to redraw the view
-   * whenever the "tree-icon-emblems" preference changes.
-   */
+  /* grab a reference on the preferences */
   view->preferences = thunar_preferences_get ();
-  g_signal_connect_swapped (G_OBJECT (view->preferences), "notify::tree-icon-emblems", G_CALLBACK (gtk_widget_queue_draw), view);
 
   /* Create a tree model for this tree view */
   view->model = g_object_new (THUNAR_TYPE_TREE_MODEL, NULL);
@@ -361,7 +358,7 @@ thunar_tree_view_init (ThunarTreeView *view)
   gtk_tree_view_set_model (GTK_TREE_VIEW (view), GTK_TREE_MODEL (view->model));
 
   /* configure the tree view */
-  gtk_tree_view_set_enable_search (GTK_TREE_VIEW (view), FALSE);
+  gtk_tree_view_set_enable_search (GTK_TREE_VIEW (view), TRUE);
   gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (view), FALSE);
 
   /* allocate a single column for our renderers */
@@ -391,6 +388,14 @@ thunar_tree_view_init (ThunarTreeView *view)
    */
   g_object_bind_property (G_OBJECT (view->preferences), "tree-icon-size", G_OBJECT (view->icon_renderer), "size", G_BINDING_SYNC_CREATE);
   g_object_bind_property (G_OBJECT (view->preferences), "tree-icon-emblems", G_OBJECT (view->icon_renderer), "emblems", G_BINDING_SYNC_CREATE);
+  g_signal_connect_swapped (G_OBJECT (view->preferences), "notify::tree-icon-emblems", G_CALLBACK (gtk_widget_queue_draw), view);
+
+  /* optional tree lines */
+  g_object_bind_property (G_OBJECT (view->preferences), "misc-tree-lines-in-tree-sidepane", G_OBJECT (view), "enable-tree-lines", G_BINDING_SYNC_CREATE);
+
+  /* optional symbolic icons */
+  g_object_bind_property (G_OBJECT (view->preferences), "misc-symbolic-icons-in-sidepane", G_OBJECT (view->icon_renderer), "use-symbolic-icons", G_BINDING_SYNC_CREATE);
+  g_signal_connect_swapped (G_OBJECT (view->preferences), "notify::misc-symbolic-icons-in-sidepane", G_CALLBACK (gtk_widget_queue_draw), view);
 
   /* allocate the text renderer */
   renderer = gtk_cell_renderer_text_new ();
@@ -458,7 +463,8 @@ thunar_tree_view_finalize (GObject *object)
   /* release reference on the action manager */
   g_object_unref (view->action_mgr);
 
-  /* release our reference on the preferences */
+  /* disconnect from the preferences object */
+  g_signal_handlers_disconnect_by_data (G_OBJECT (view->preferences), view);
   g_object_unref (G_OBJECT (view->preferences));
 
   /* free the tree model */
@@ -1149,6 +1155,10 @@ thunar_tree_view_row_activated (GtkTreeView       *tree_view,
           thunar_tree_view_action_open (THUNAR_TREE_VIEW (tree_view));
         }
     }
+
+  /* Using TREE_SEARCH and <Return> opens a folder, but also our treeview
+   * looses the focus. Get the focus back: */
+  gtk_widget_grab_focus (GTK_WIDGET (tree_view));
 }
 
 
@@ -1276,13 +1286,12 @@ thunar_tree_view_context_menu (ThunarTreeView *view,
         }
       else
         {
-          thunar_menu_add_sections (context_menu, THUNAR_MENU_SECTION_OPEN);
-          thunar_menu_add_sections (context_menu, THUNAR_MENU_SECTION_SENDTO
-                                                | THUNAR_MENU_SECTION_CREATE_NEW_FILES
+          thunar_menu_add_sections (context_menu, THUNAR_MENU_SECTION_OPEN
+                                                | THUNAR_MENU_SECTION_SENDTO);
+          thunar_menu_add_sections (context_menu, THUNAR_MENU_SECTION_CREATE_NEW_FILES
                                                 | THUNAR_MENU_SECTION_CUT
                                                 | THUNAR_MENU_SECTION_COPY_PASTE
                                                 | THUNAR_MENU_SECTION_TRASH_DELETE
-                                                | THUNAR_MENU_SECTION_RESTORE
                                                 | THUNAR_MENU_SECTION_RENAME
                                                 | THUNAR_MENU_SECTION_CUSTOM_ACTIONS);
         }
