@@ -20,7 +20,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #ifdef HAVE_SYS_TYPES_H
@@ -52,8 +52,8 @@
 
 #include <libxfce4util/libxfce4util.h>
 
-#include <thunar/thunar-user.h>
-#include <thunar/thunar-util.h>
+#include "thunar/thunar-user.h"
+#include "thunar/thunar-util.h"
 
 
 
@@ -561,8 +561,6 @@ thunar_user_manager_flush_timer (gpointer user_data)
   ThunarUserManager *manager = THUNAR_USER_MANAGER (user_data);
   guint              size = 0;
 
-THUNAR_THREADS_ENTER
-
   /* drop all cached groups */
   size += g_hash_table_foreach_remove (manager->groups,
                                        (GHRFunc) (void (*)(void)) gtk_true,
@@ -588,8 +586,6 @@ THUNAR_THREADS_ENTER
 #endif
     }
 
-THUNAR_THREADS_LEAVE
-
   return TRUE;
 }
 
