@@ -54,7 +54,8 @@ typedef enum
   THUNAR_WINDOW_ACTION_PREFERENCES,
   THUNAR_WINDOW_ACTION_VIEW_MENU,
   THUNAR_WINDOW_ACTION_RELOAD,
-  THUNAR_WINDOW_ACTION_RELOAD_ALT,
+  THUNAR_WINDOW_ACTION_RELOAD_ALT_1,
+  THUNAR_WINDOW_ACTION_RELOAD_ALT_2,
   THUNAR_WINDOW_ACTION_VIEW_SPLIT,
   THUNAR_WINDOW_ACTION_SWITCH_FOCUSED_SPLIT_VIEW_PANE,
   THUNAR_WINDOW_ACTION_VIEW_LOCATION_SELECTOR_MENU,
@@ -84,8 +85,10 @@ typedef enum
   THUNAR_WINDOW_ACTION_BOOKMARKS_MENU,
   THUNAR_WINDOW_ACTION_OPEN_PARENT,
   THUNAR_WINDOW_ACTION_BACK,
-  THUNAR_WINDOW_ACTION_BACK_ALT,
+  THUNAR_WINDOW_ACTION_BACK_ALT_1,
+  THUNAR_WINDOW_ACTION_BACK_ALT_2,
   THUNAR_WINDOW_ACTION_FORWARD,
+  THUNAR_WINDOW_ACTION_FORWARD_ALT,
   THUNAR_WINDOW_ACTION_OPEN_FILE_SYSTEM,
   THUNAR_WINDOW_ACTION_OPEN_HOME,
   THUNAR_WINDOW_ACTION_OPEN_DESKTOP,
@@ -102,6 +105,7 @@ typedef enum
   THUNAR_WINDOW_ACTION_SWITCH_PREV_TAB,
   THUNAR_WINDOW_ACTION_SWITCH_NEXT_TAB,
   THUNAR_WINDOW_ACTION_SEARCH,
+  THUNAR_WINDOW_ACTION_SEARCH_ALT,
   THUNAR_WINDOW_ACTION_CANCEL_SEARCH,
   THUNAR_WINDOW_ACTION_SHOW_HIGHLIGHT,
 
@@ -130,6 +134,7 @@ void                      thunar_window_notebook_remove_tab                 (Thu
                                                                              gint                 tab);
 void                      thunar_window_notebook_set_current_tab            (ThunarWindow        *window,
                                                                              gint                 tab);
+void                      thunar_window_paned_notebooks_switch              (ThunarWindow        *window);
 gboolean                  thunar_window_has_shortcut_sidepane               (ThunarWindow        *window);
 gboolean                  thunar_window_has_tree_view_sidepane              (ThunarWindow        *window);
 GtkWidget*                thunar_window_get_sidepane                        (ThunarWindow        *window);
