@@ -20,13 +20,13 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <libxfce4util/libxfce4util.h>
 
-#include <thunarx/thunarx-file-info.h>
-#include <thunarx/thunarx-private.h>
+#include "thunarx/thunarx-file-info.h"
+#include "thunarx/thunarx-private.h"
 
 
 
