@@ -19,15 +19,15 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #include <glib/gi18n-lib.h>
 
 #include <libxfce4util/libxfce4util.h>
 
-#include <thunarx/thunarx-renamer.h>
-#include <thunarx/thunarx-private.h>
+#include "thunarx/thunarx-renamer.h"
+#include "thunarx/thunarx-private.h"
 
 
 
